'''
   CheckBoxs
     Show array of CheckBoxs to ask user to select 
     items.

     checkbox_panel(parent, col = 4, 
                   labels = ('label 1', 'label 2', 'label 3'),
                   values = (False, False, True),
                   message = 'head line message',
                   title = 'dialog title'):


     return value [(label1, True/False), ...]

'''
import wx
import math
import ifigure.utils.cbook as cbook
from ifigure.utils.wx3to4 import GridSizer


class CheckBoxs(wx.Panel):
    def __init__(self, *args, **kargs):
        setting = kargs['setting']
        del kargs['setting']
        wx.Panel.__init__(self, *args, **kargs)

        labels = setting['labels']
        row = math.ceil(float(len(labels))/setting['col'])

        sizer = GridSizer(row, setting['col'])
        self.SetSizer(sizer)

        self.child = []
        for k, label in enumerate(labels):
            p = wx.CheckBox(self, wx.ID_ANY, label)
            self.child.append(p)
            r = k/setting['col']
            c = k - r*setting['col']
#           print r, c
            sizer.Add(p, 1, wx.EXPAND)
            p.Bind(wx.EVT_RIGHT_UP, self.onRightUp)
            p.Bind(wx.EVT_CHECKBOX, self.onHitCheck)
        self.Layout()
        self.labels = labels
        self._init = False

    def GetValue(self):
        return [(l, c.GetValue()) for l, c in zip(self.labels, self.child)]

    def SetValue(self, values):
        for k, v in enumerate(values):
            self.child[k].SetValue(v)
        if not self._init:
            self._init = True
            self._init_values = values

    def SelAll(self, evt=None):
        self.SetValue([True]*len(self.child))

    def UnselAll(self, evt=None):
        self.SetValue([False]*len(self.child))

    def ResetChecks(self, evt=None):
        self.SetValue(self._init_values)

    def onHitCheck(self, evt):
        evt.SetEventObject(self)
        if hasattr(self.GetParent(), "send_event"):
            self.GetParent().send_event(self, evt)
        
    def onRightUp(self, evt):
        l = [('Select all',   self.SelAll, None),
             ('Clear all',    self.UnselAll, None), ]
        menu = wx.Menu()
        f1 = menu.Append(wx.ID_ANY, 'Select All')
        self.Bind(wx.EVT_MENU, self.SelAll, f1)
        f2 = menu.Append(wx.ID_ANY, 'Clear All', '')
        self.Bind(wx.EVT_MENU, self.UnselAll, f2)
        f3 = menu.Append(wx.ID_ANY, 'Reset', '')
        self.Bind(wx.EVT_MENU, self.ResetChecks, f3)
        evt.GetEventObject().PopupMenu(menu, evt.GetPosition())
        menu.Destroy()
        evt.Skip()


CheckBoxes = CheckBoxs  # patch work to fix misspell ;D


class CheckboxsDialog(wx.Dialog):
    def __init__(self, parent, id=-1, title="CheckBoxDialg", col=4,
                 labels=['A', 'B', 'C'], values=None,
                 message=None, elp=None, blabel='OK'):
        if values is None:
            values = [False]*len(labels)
        self.ovalues = values

        wx.Dialog.__init__(self, parent, id, title)

        mainSizer = wx.BoxSizer(wx.VERTICAL)
        sizer = wx.BoxSizer(wx.HORIZONTAL)

        setting = {'col': col,
                   'labels': labels}
        if message is not None:
            m = wx.StaticText(self, wx.ID_ANY, message)
            mainSizer.Add(m,  0, wx.ALL, 0)
        self.checkboxs = CheckBoxs(self, wx.ID_ANY, setting=setting)
        sizer.Add(self.checkboxs,  1, wx.ALL | wx.EXPAND, 0)

        self.buttonSizer = wx.BoxSizer(wx.HORIZONTAL)
        selall = wx.Button(self, label="Select All", id=wx.ID_ANY)
        selall.Bind(wx.EVT_BUTTON, self.onSelAll)
        unselall = wx.Button(self, label="Unselect All", id=wx.ID_ANY)
        unselall.Bind(wx.EVT_BUTTON, self.onUnselAll)
        reset = wx.Button(self, label="Reset", id=wx.ID_ANY)
        reset.Bind(wx.EVT_BUTTON, self.onReset)
        self.okbutton = wx.Button(self, label=blabel, id=wx.ID_OK)
        self.Bind(wx.EVT_BUTTON, self.onOK, id=wx.ID_OK)

        self.buttonSizer.Add(reset, 1, wx.ALL | wx.EXPAND, 8)
        self.buttonSizer.Add(selall, 1, wx.ALL | wx.EXPAND, 8)
        self.buttonSizer.Add(unselall, 1, wx.ALL | wx.EXPAND, 8)
        self.buttonSizer.Add(self.okbutton, 1, wx.ALL | wx.EXPAND, 8)

        mainSizer.Add(sizer,  1, wx.ALL | wx.EXPAND, 5)
        mainSizer.Add(self.buttonSizer, 0, wx.ALL, 0)
        self.SetSizer(mainSizer)
        self.Fit()
        self.Centre()
        self.result = None
        self.SetValue(values)
        wx.CallAfter(self._myRefresh)

    def onSelAll(self, event):
        self.checkboxs.SelAll()

    def onUnselAll(self, event):
        self.checkboxs.UnselAll()

    def onReset(self, event):
        self.checkboxs.SetValue(self.ovalues)

    def onOK(self, event):
        self.result = self.GetValue()
        if self.IsModal():
            self.EndModal(wx.ID_OK)
        else:
            self.Destroy()

    def SetValue(self, value):
        self.checkboxs.SetValue(value)

    def GetValue(self):
        return self.checkboxs.GetValue()

    def _myRefresh(self):
        win = self.GetTopLevelParent()
#        win.SetSizeHints(win)
        win.Fit()
        win.Layout()


def checkbox_panel(parent, col=4,
                   labels=('label 1', 'label 2', 'label 3',
                           'label 4', 'label 5'),
                   values=(False, False, True, True, True),
                   message=None, title='', blabel='OK'):

    dig = CheckboxsDialog(parent, labels=labels, values=values, message=message,
                          title=title, col=col, blabel=blabel)
    dig.SetValue(values)
    a = dig.ShowModal()
    if a == wx.ID_OK:
        v = dig.GetValue()
    else:
        return None
    dig.Destroy()
    return v



# triplot_mod
###
# modified to return artists

from matplotlib.cbook import ls_mapper
from matplotlib.patches import PathPatch
from matplotlib.path import Path
from matplotlib.tri import Triangulation
import numpy as np


def _process_plot_format(fmt):
    """
    Process a MATLAB style color/line style format string.  Return a
    (*linestyle*, *color*) tuple as a result of the processing.  Default
    values are ('-', 'b').  Example format strings include:

    * 'ko': black circles
    * '.b': blue dots
    * 'r--': red dashed lines

    .. seealso::

        :func:`~matplotlib.Line2D.lineStyles` and
        :func:`~matplotlib.pyplot.colors`
            for all possible styles and color format string.
    """
    import matplotlib
    rcParams = matplotlib.rcParams

    import matplotlib.colors as mcolors

    linestyle = None
    marker = None
    color = None

    # Is fmt just a colorspec?
    try:
        color = mcolors.colorConverter.to_rgb(fmt)

        # We need to differentiate grayscale '1.0' from tri_down marker '1'
        try:
            fmtint = str(int(fmt))
        except ValueError:
            return linestyle, marker, color  # Yes
        else:
            if fmt != fmtint:
                # user definitely doesn't want tri_down marker
                return linestyle, marker, color  # Yes
            else:
                # ignore converted color
                color = None
    except ValueError:
        pass  # No, not just a color.

    # handle the multi char special cases and strip them from the
    # string
    if fmt.find('--') >= 0:
        linestyle = '--'
        fmt = fmt.replace('--', '')
    if fmt.find('-.') >= 0:
        linestyle = '-.'
        fmt = fmt.replace('-.', '')
    if fmt.find(' ') >= 0:
        linestyle = 'None'
        fmt = fmt.replace(' ', '')

    chars = [c for c in fmt]

    for c in chars:
        if c in mlines.lineStyles:
            if linestyle is not None:
                raise ValueError(
                    'Illegal format string "%s"; two linestyle symbols' % fmt)
            linestyle = c
        elif c in mlines.lineMarkers:
            if marker is not None:
                raise ValueError(
                    'Illegal format string "%s"; two marker symbols' % fmt)
            marker = c
        elif c in mcolors.colorConverter.colors:
            if color is not None:
                raise ValueError(
                    'Illegal format string "%s"; two color symbols' % fmt)
            color = c
        else:
            raise ValueError(
                'Unrecognized character %c in format string' % c)

    if linestyle is None and marker is None:
        linestyle = rcParams['lines.linestyle']
    if linestyle is None:
        linestyle = 'None'
    if marker is None:
        marker = 'None'

    return linestyle, marker, color


def triplot(ax, *args, **kwargs):
    """
    Draw a unstructured triangular grid as lines and/or markers to
    the :class:`~matplotlib.axes.Axes`.

    The triangulation to plot can be specified in one of two ways;
    either::

      triplot(triangulation, ...)

    where triangulation is a :class:`~matplotlib.tri.Triangulation`
    object, or

    ::

      triplot(x, y, ...)
      triplot(x, y, triangles, ...)
      triplot(x, y, triangles=triangles, ...)
      triplot(x, y, mask=mask, ...)
      triplot(x, y, triangles, mask=mask, ...)

    in which case a Triangulation object will be created.  See
    :class:`~matplotlib.tri.Triangulation` for a explanation of these
    possibilities.

    The remaining args and kwargs are the same as for
    :meth:`~matplotlib.axes.Axes.plot`.

    **Example:**

        .. plot:: mpl_examples/pylab_examples/triplot_demo.py
    """
    import matplotlib.axes
    tri, args, kwargs = Triangulation.get_from_args_and_kwargs(*args, **kwargs)

    x = tri.x
    y = tri.y
    edges = tri.edges

    # If draw both lines and markers at the same time, e.g.
    #     ax.plot(x[edges].T, y[edges].T, *args, **kwargs)
    # then the markers are drawn more than once which is incorrect if alpha<1.
    # Hence draw lines and markers separately.

    # Decode plot format string, e.g. 'ro-'
    fmt = ''
    if len(args) > 0:
        fmt = args[0]

#   _process_plot_format moves around so I made copy here.
#   not a best solution...;D
#    linestyle, marker, color = matplotlib.axes._process_plot_format(fmt)
    linestyle, marker, color = _process_plot_format(fmt)

    # Draw lines without markers, if lines are required.
    a = []
    if linestyle is not None and linestyle != 'None':
        kw = kwargs.copy()
        kw.pop('marker', None)     # Ignore marker if set.
        kw['linestyle'] = ls_mapper[linestyle]
        kw['edgecolor'] = color
        kw['facecolor'] = None

        vertices = np.column_stack((x[edges].flatten(), y[edges].flatten()))
        codes = ([Path.MOVETO] + [Path.LINETO])*len(edges)

        path = Path(vertices, codes)
        pathpatch = PathPatch(path, **kw)

        ax.add_patch(pathpatch)
        a.append(pathpatch)

    # Draw markers without lines.
    # Should avoid drawing markers for points that are not in any triangle?
    kwargs['linestyle'] = ''

    # without hiding points explicitly, marker would expose hidden points.
    idx = np.unique(edges.flatten())
    l = ax.plot(x[idx], y[idx], *args, **kwargs)
    a = l+a
    return a



import numpy as np
######################################################
#         Setting for module file for py_module
#
#   General rule:
#      This file will be automatically loaded when
#      py_module object is created. Also, py_module
#      keeps track the file modification time. If
#      the file is updated, it will be automaticaaly
#      reloaded.
#      Strong recommendation : make module "independent".
#      Py_Modules does not check the dependency of
#      modules.
#      If moduels used in Py_Modules depends on
#      each other by for example module variable,
#      it will cause complicate  module loading
#      order-dependency problem.
#
#   name of module/class
module_name = 'cmod_pyro'
class_name = 'cmod_pyro'
#   module_evt_handler
#   functions which can be called from project tree
#
#    (menu name, function name, a flat to call skip()
#     after running function)
#
#   By default these function should return None
#   or True
#   if it return False, ifigure stops exectuion at
#   this module
#
menu = [("Launch NS", "onLaunchNS", True),
        ("Launch DM", "onLaunchDM", True)]


method = ['init', 'clean', 'onLaunchNS', 'onLaunchDM']

icon = 'world_link.png'
can_have_child = False
has_private_owndir = False
######################################################


def init(self):
    self.td.setvar("daemons",
                   (("proxy_efit", True, "cmodws60.psfc.mit.edu", -1),
                    ("proxy_nete", True, "cmodws60.psfc.mit.edu", -1)))
    self.td.setvar("pyronshost", "cmodws30.psfc.mit.edu")


def clean(self):
    # clean is a special process called when td is deleted
    if hasattr(self, 'p'):
        self.p.terminate()
        del self.p


def onLaunchNS(self, e):
    pass


def onLaunchDM(self, e):
    pass



import wx
import sys
import time
import wx.aui as aui
import ifigure
import ifigure.events


class Logwindow(wx.MiniFrame):
    def __init__(self, *args, **kargs):
        self.threadlist = []
        # this is added not to have windows "always on parent"
        #        args2 = [x for x in args]
        #        args2[0] = None
        #        args = tuple(args2)
        ###
        wx.MiniFrame.__init__(self, *args,
                              style=wx.CAPTION |
                              wx.CLOSE_BOX |
                              wx.MINIMIZE_BOX |
                              wx.RESIZE_BORDER |
                              wx.FRAME_FLOAT_ON_PARENT)
        panel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vbox)
        vbox.Add(panel, 1, wx.EXPAND | wx.ALL, 5)

        self.nb = aui.AuiNotebook(panel)

        bpanel = wx.Panel(panel)
        button2 = wx.Button(bpanel, wx.ID_ANY, "Close Finished Thread Tab")
        button = wx.Button(bpanel, wx.ID_ANY, "Close")

        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)

        panel.SetSizer(vbox)
        vbox.Add(self.nb, 1, wx.EXPAND)
        vbox.Add(bpanel, 0, wx.EXPAND | wx.ALL, 5)

        hbox.AddStretchSpacer()
        hbox.Add(button2, 0, wx.EXPAND | wx.ALL, 2)
        hbox.Add(button, 0, wx.EXPAND | wx.ALL, 2)
        bpanel.SetSizer(hbox)

        #sys.stdout = RedirectOutput(self.log)
        #sys.stderr = RedirectOutput(self.log)

        button.Bind(wx.EVT_BUTTON, self.onPanelClose)
        button2.Bind(wx.EVT_BUTTON, self.onCloseFinished)
        self.Bind(wx.EVT_CLOSE, self.onWindowClose)
        self.Bind(wx.aui.EVT_AUINOTEBOOK_PAGE_CLOSE,
                  self.onPageClose)

        self.Layout()
        self.Centre()
        self.Hide()

        self.redirector = None
        #self.timer = wx.Timer(self)
        self.Bind(wx.EVT_TIMER, self.check_thread)
        self.Bind(wx.EVT_ACTIVATE, self.onActivate)

    def set_redirector(self, redir):
        self.redirector = redir

    def watch_thread(self, evt):
        #        print 'watch thread', evt.thread
        # ev = ThreadStart event
        td = evt.GetTreeDict()
        log = wx.TextCtrl(self.nb, -1,
                          style=wx.TE_MULTILINE | wx.TE_READONLY)
        self.nb.AddPage(log, evt.thread.name)
        self.redirector.add(evt.thread, log.AppendText)
        self.threadlist.append([evt.thread, td, log, 0])

        wx.CallLater(1000, self.check_thread)

        st = time.localtime(time.time())
        strt = time.strftime(" %b %d, %Y  %I:%M:%S", st)
        log.AppendText('thread registered at '+strt+'\n')

    def start_thread(self):
        c = 0
        for t, td, log, status in self.threadlist:
            if t.is_alive():
                c = c+1
        if c > self.GetParent().aconfig.setting['max_thread']:
            return

        i = 0
        for t, td, log, status in self.threadlist:
            if status == 0:
                t.start()
                # this is when we want to run the process in the main thread temtativcely...
                #t.run()
                st = time.localtime(time.time())
                strt = time.strftime(" %b %d, %Y  %I:%M:%S", st)
                log.AppendText('thread started at '+strt+'\n')
                self.threadlist[i][3] = 1
                break
            i = i+1

    def check_thread(self, evt=None):
        #        print 'check thread'
        restart = False
        deadlist = []
        waiting = False
        num_run = 0
        for t, td, log, status in self.threadlist:
            if t.is_alive() or status == 0:
                if status == 0:
                    waiting = True
                else:
                    num_run = num_run + 1
                restart = True
            else:
                td._status = ''
                ifigure.events.SendChangedEvent(td)
                st = time.localtime(time.time())
                strt = time.strftime(" %b %d, %Y  %I:%M:%S", st)
                log.AppendText('done....('+strt+')\n')
                self.redirector.rm(t)
                deadlist.append(t)

        for t in deadlist:
            self.threadlist = [x for x
                               in self.threadlist if x[0] != t]
        if restart:
            #print('restarting log window timer')
            wx.CallLater(1000, self.check_thread)
            #self.timer.Start(1000, oneShot=True)

        if waiting:
            if (num_run <
                    self.GetParent().aconfig.setting['max_thread']):
                self.start_thread()

    def check_thread_can_start(self, t):
        return True

    def onCloseFinished(self, e=None):
        ipage = self.nb.GetSelection()
        n = self.nb.GetPageCount()
        l = [True]*n
        for ipage in reversed(range(n)):
            p = self.nb.GetPage(ipage)

            keep = False
            for x in self.threadlist:
                if x[2] == p:
                    keep = True
            if not keep:

                self.nb.RemovePage(ipage)
                p.Destroy()

        return

    def onActivate(self, e):
        if e.GetActive():
            self.SetTransparent(255)
        else:
            self.SetTransparent(120)

    def onPanelClose(self, e=None):
        self.Hide()

    def onWindowClose(self, e=None):
        self.Hide()
        e.Veto()

    def call_remove_page(self, ipage, cpage):
        if (self.nb.GetPageCount() == cpage):
            self.nb.RemovePage(ipage)

    def onPageClose(self, e=None):
        ipage = self.nb.GetSelection()
        p = self.nb.GetPage(ipage)
        wx.CallAfter(self.call_remove_page, ipage, self.nb.GetPageCount())
        i = 0
        for x in self.threadlist:
            if x[2] == p:
                self.redirector.rm(x[0])
        self.threadlist = [x for x
                           in self.threadlist if x[2] != p]



from __future__ import print_function
#
#  fig_eps
#
#     This class is to place raster image generated fro
#     eps file
#     It will keep eps file and embed it as vector
#     data when exporting to eps/pdf format
#
#     it uses convert for raster image generation.
#     ps2pdf -dEPSCrop is used for pdf genration.
#     pdfrw is used to Form XObject generation.
#
#     It create a FigImageV object which is essentially
#     FigImage, but draw is overwritten so that when
#     renderer is RendererPS or RenderePDF, it bypass
#     the orignal routine and add eps/pdf to a picutre
#     file.
#
#     FigImageV is placed in figure.artists, not figure.images
#     in order to avoid image decomposition in figure::draw
#
#  History:
#         2013 12
#  Author :
#         Syun'ichi Shiraiwa
#  E-mail :
#         shiraiwa@psfc.mit.edu
#
# *******************************************
#     Copyright(c) 2012- PSFC, MIT
# *******************************************
from matplotlib.backends.backend_pdf import RendererPdf, PdfFile, Name, Op, pdfRepr
import matplotlib.backends.backend_ps
from matplotlib.backends.backend_ps import RendererPS
import six
import weakref
import traceback

from ifigure.mto.fig_obj import FigObj
from ifigure.mto.axis_user import XUser, YUser, CUser
from ifigure.widgets.canvas.file_structure import *
import ifigure
import os
import sys
import shutil
from six import itervalues, iteritems
import subprocess as sp
import numpy as np
import ifigure.utils.cbook as cbook
import ifigure.widgets.canvas.custom_picker as cpicker
from scipy.interpolate import griddata
from ifigure.utils.cbook import ProcessKeywords
import matplotlib
from matplotlib.tri import Triangulation
from matplotlib.cm import ScalarMappable
import matplotlib.image as mpimage
from ifigure.utils.args_parser import ArgsParser
from ifigure.mto.fig_box import FigBox
from matplotlib.image import FigureImage
import ifigure.utils.debug as debug
debug.debug_default_level = 1
dprint1, dprint2, dprint3 = debug.init_dprints('FigEPS')


def calc_newpos_from_anchor(p1, p2, dp, mode):
    if mode == 0:
        return (p1, p1+dp)
    elif mode == 1:
        m = (p1+p2)/2.
        return (m-dp/2., m+dp/2.)
    else:
        return (p2-dp, p2)


unicode_file = 'unicode_literals' in dir(matplotlib.backends.backend_ps)

eps_funcs = '\n'.join(['/BeginEPSF {',
                       '  /EPSFsave save def',
                       '  count /OpStackSize exch def',
                       '  /DictStackSize countdictstack def',
                       '  % turn off showpage',
                       '  /showpage {} def',
                       '  % set up default graphics state',
                       '  0 setgray 0 setlinecap',
                       '  1 setlinewidth 0 setlinejoin',
                       '  10 setmiterlimit [] 0 setdash newpath',
                       '  /languagelevel where',
                       '  {pop languagelevel 1 ne',
                       '    {false setstrokeadjust false setoverprint} if',
                       '  } if',
                       '} bind def',
                       '/EndEPSF {',
                       '  count OpStackSize sub',
                       '  dup 0 lt {neg {pop} repeat} {pop} ifelse',
                       '  countdictstack DictStackSize sub ',
                       '  dup 0 lt {neg {end} repeat} {pop} ifelse',
                       '  EPSFsave restore',
                       '} bind def\n'])


class PdfFile_plus(PdfFile):
    def close(self):
        print('PdfFile plus is closing file')
        self.endStream()
        # Write out the various deferred objects
        self.writeFonts()
        self.writeObject(self.alphaStateObject,
                         dict([(val[0], val[1])
                               for val in itervalues(self.alphaStates)]))
        self.writeHatches()
        self.writeGouraudTriangles()
        xobjects = dict(iter(itervalues(self.images)))
        for tup in itervalues(self.markers):
            xobjects[tup[0]] = tup[1]
        for name, value in iteritems(self.multi_byte_charprocs):
            xobjects[name] = value
        for name, path, trans, ob, join, cap, padding, filled, stroked in self.paths:
            xobjects[name] = ob
        for tup in itervalues(self.pdfs):
            xobjects[tup[0]] = tup[1]
        self.writeObject(self.XObjectObject, xobjects)
        self.writeImages()
        self.writeMarkers()
        self.writePathCollectionTemplates()
        self.writePdfs()
        self.writeObject(self.pagesObject,
                         {'Type': Name('Pages'),
                          'Kids': self.pageList,
                          'Count': len(self.pageList)})
        self.writeInfoDict()

        # Finalize the file
        self.writeXref()
        self.writeTrailer()
        if self.passed_in_file_object:
            self.fh.flush()
        elif self.original_file_like is not None:
            self.original_file_like.write(self.fh.getvalue())
            self.fh.close()
        else:
            self.fh.close()

    def pdfObject(self, pdf):
        """Return name of an pdf XObject representing the given pdf."""
        pair = self.pdfs.get(id(pdf), None)
        if pair is not None:
            return pair[1]

        name = Name('Pdf%d' % self.nextPdf)
        ob = self.reserveObject('pdf %d' % self.nextPdf)
        self.nextPdf += 1
        self.pdfs[id(pdf)] = (pdf, name, ob)

        return name

    def pdfResources(self, d, pairs=None):
        from pdfrw.objects import PdfName, PdfArray, PdfDict, IndirectPdfDict, PdfObject
        if pairs is None:
            pairs = []
        v = dict()
        for key in d:
            if not isinstance(d[key], PdfDict):
                #                if isinstance(d[key], str):
                #                    v[key[1:]] = d[key][1:]
                #                else:
                v[key[1:]] = self.pdfobj2value(d[key])
            else:
                id = self.reserveObject('pdf resource element')
                v[key[1:]] = id
                pairs.append((id, d[key]))
        return v, pairs

    def pdfobj2value(self, obj):
        from pdfrw.objects import PdfName, PdfArray, PdfDict, IndirectPdfDict, PdfObject

        def convert(m):
            if str(m).startswith('/'):
                return Name(str(m)[1:])
            elif str(m) in ['false', 'true']:
                if str(m) == 'false':
                    return False
                else:
                    return True
            else:
                return eval(m)
        if isinstance(obj, PdfArray):
            return [convert(x) for x in obj]
        else:
            return convert(obj)

    def writeObject_stream(self, object, contents, stream):
        self.recordXref(object.id)
        self.write(("%d 0 obj\n" % object.id).encode('ascii'))
        self.write(pdfRepr(contents))
        self.write(b"\nstream\n")
        self.write(np.fromstring(stream, np.uint8))
        self.write(b"\nendstream\nendobj\n")

    def writePdfResources(self, pairs):
        from pdfrw.objects import PdfName, PdfArray, PdfDict, IndirectPdfDict, PdfObject
        next_pairs = []
        for id, value in pairs:
            if isinstance(value, PdfDict):
                d = {}
                pairs = []
                for key in value:
                    if isinstance(value[key], PdfDict):
                        d[key[1:]] = self.reserveObject('pdf resource element')
                        pairs.append((d[key[1:]], value[key]))
                    else:
                        d[key[1:]] = self.pdfobj2value(value[key])
                if value.stream is not None:
                    self.writeObject_stream(id, d, value.stream)
                else:
                    self.writeObject(id, d)
                next_pairs.append(pairs)
            else:
                if value.stream is not None:
                    self.writeObject_stream(
                        id, self.pdfobj2value(value), value.stream)
                else:
                    self.writeObject(id, self.pdfobj2value(value))

        for pairs in next_pairs:
            self.writePdfResources(pairs)

    def writePdfs(self):
        for pdf, name, ob in six.itervalues(self.pdfs):
            xobj = pdf.get_xobj()
            dd = {'Type': Name('XObject'), 'Subtype': Name('Form'),
                  'BBox': xobj.BBox, 'FormType': xobj.FormType,
                  'Filter': xobj.Filter, }
            if '/Resources' in xobj:
                resources, id_pairs = self.pdfResources(xobj.Resources)
                dd['Resources'] = resources
            self.beginStream(
                ob.id,
                self.reserveObject('length of pdf stream'),
                dd)
            self.currentstream.compressobj = None
            self.currentstream.write(np.fromstring(xobj.stream, np.uint8))
            self.endStream()
            if '/Resources' in xobj:
                self.writePdfResources(id_pairs)


class PdfFile_plus2(PdfFile_plus):
    def close(self):
        print('PdfFile plus2 is closing file')
        self.endStream()
        # Write out the various deferred objects
        self.writeFonts()
        self.writeObject(self.alphaStateObject,
                         dict([(val[0], val[1])
                               for val in six.itervalues(self.alphaStates)]))
        self.writeHatches()
        self.writeGouraudTriangles()
        xobjects = dict(x[1:] for x in six.itervalues(self._images))
        for tup in six.itervalues(self.markers):
            xobjects[tup[0]] = tup[1]
        for name, value in six.iteritems(self.multi_byte_charprocs):
            xobjects[name] = value
        for name, path, trans, ob, join, cap, padding, filled, stroked \
                in self.paths:
            xobjects[name] = ob
        for pdf, name, ob in six.itervalues(self.pdfs):
            xobjects[name] = ob
        self.writeObject(self.XObjectObject, xobjects)
        self.writeImages()
        self.writeMarkers()
        self.writePathCollectionTemplates()
        self.writePdfs()
        self.writeObject(self.pagesObject,
                         {'Type': Name('Pages'),
                          'Kids': self.pageList,
                          'Count': len(self.pageList)})
        self.writeInfoDict()

        # Finalize the file
        self.writeXref()
        self.writeTrailer()
        if self.passed_in_file_object:
            self.fh.flush()
        elif self.original_file_like is not None:
            self.original_file_like.write(self.fh.getvalue())
            self.fh.close()
        else:
            self.fh.close()


class RendererPdf_plus(RendererPdf):
    def draw_pdf(self, gc, x, y, scale, o):
        self.check_gc(gc)

        pdfob = self.file.pdfObject(o)
        self.file.output(Op.gsave, scale[0], 0, 0, scale[1], x, y,
                         Op.concat_matrix,
                         pdfob, Op.use_xobject, Op.grestore)


def pdffile_upgrade(pdffile):
    from ifigure.ifigure_config import isMPL2
    if isMPL2:
        cls = PdfFile_plus2
    else:
        cls = PdfFile_plus
    if pdffile.__class__ != cls:
        pdffile.__class__ = cls
        pdffile.pdfs = {}
        pdffile.nextPdf = 1


def mixedrenderer_upgrade(renderer):
    if renderer._vector_renderer.__class__ != RendererPdf_plus:
        renderer._vector_renderer.__class__ = RendererPdf_plus
        setattr(renderer, 'draw_pdf', getattr(
            renderer._vector_renderer, 'draw_pdf'))


class pdfobj(object):
    def __init__(self, xobj):
        self._xobj = xobj

    def get_xobj(self):
        return self._xobj


class FigureImageV(FigureImage):
    def draw(self, renderer, *args, **kwargs):
        #        return FigureImage.draw(self, renderer, *args, **kwargs)
        if (hasattr(renderer, '_vector_renderer') and
                isinstance(renderer._vector_renderer, RendererPdf)):
            pdffile_upgrade(renderer._vector_renderer.file)
            mixedrenderer_upgrade(renderer)
            gc = renderer.new_gc()
            import ifigure.utils.buildxobj as buildxobj
            o = buildxobj.CacheXObj()

            gc.set_alpha(self.get_alpha())

            ocwd = os.getcwd()
            _pdffile = os.path.join(self._figobj().owndir(), self._pdf_file)
            os.chdir(os.path.dirname(_pdffile))
            file = os.path.basename(_pdffile)
            obj = o.load(file)
            os.chdir(ocwd)
            self._obj = obj
            renderer.draw_pdf(gc, round(self.ox),
                              round(self.oy),
                              self._image_scale, pdfobj(obj))
            gc.restore()
        elif (hasattr(renderer, '_vector_renderer') and
              isinstance(renderer._vector_renderer, RendererPS)):
            gc = renderer.new_gc()
            ps_write0 = renderer._vector_renderer._pswriter.write
            _eps_file = os.path.join(self._figobj().owndir(), self._eps_file)

            def ps_write(txt):
                if unicode_file:
                    ps_write0(unicode(txt))
                else:
                    ps_write0(txt)

            if not hasattr(renderer._vector_renderer, '_eps_func_written'):
                ps_write(eps_funcs)
                renderer._vector_renderer._eps_func_written = True
            ps_write('BeginEPSF\n')
            ps_write(str(round(self.ox)) + ' ' +
                     str(round(self.oy)) + ' translate\n')
            ps_write(str(self._image_scale[0])+' ' +
                     str(self._image_scale[1]) + ' scale\n')
            ps_write(str(-self._eps_bbox[0]) + ' ' +
                     str(-self._eps_bbox[1]) + ' translate\n')
            ps_write('%%BeginDocument: "' +
                     os.path.basename(_eps_file) + '"\n')
            fid = open(_eps_file, 'r')
            for l in fid.readlines():
                ps_write(l)
            fid.close()
            ps_write('%%EndDocument\n')
            ps_write('EndEPSF\n')
            gc.restore()
        else:
            return FigureImage.draw(self, renderer, *args, **kwargs)


def create_figimagev(figure, X,
                     xo=0,
                     yo=0,
                     alpha=None,
                     norm=None,
                     cmap=None,
                     vmin=None,
                     vmax=None,
                     origin=None,
                     **kwargs):
    '''
    add figimagev object to figure. Made based on figure::figimage
    '''
    im = FigureImageV(figure, cmap, norm, xo, yo, origin, **kwargs)
    im.set_array(X)
    im.set_alpha(alpha)
    if norm is None:
        im.set_clim(vmin, vmax)
#   figure.images.append(im)
    figure.artists.append(im)
    return im


class FigEPS(FigBox):
    def __new__(cls, *args, **kywds):
        def set_hidden_vars(obj):
            obj._image_size = (-1, -1)
            obj._image_scale_str = ('100', '100')
            obj._eps_bbox = (-1, -1, -1, -1)
            obj._image = None
            obj._image_artists = []
            obj._image_alpha = 1.0
            obj._image_scale = (1, 1)
            obj._keep_aspect = True
            # 0 (left or bottom) 1 (center) 2 (right, top)
            obj._anchor_mode = (0, 0)
            obj._resize_mode = True  # True  use normalized coords.
            # False evaulate new size from the eps bbox
            return obj

        p = ArgsParser()
        p.add_opt('org_epsfile', None, 'str')
        p.add_opt('xy', [0.2, 0.2], 'numbers')

        v, kywds, d, flag = p.process(*args, **kywds)
        if not flag:
            raise ValueError('Failed when processing argument')

        obj = FigObj.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)

        for name in ('xy', 'org_epsfile'):
            obj.setvar(name, v[name])
        obj.setvar("kywds", kywds)
        return obj

    def __init__(self, *args, **kywds):
        if len(args) > 0:
            file = args[0]
        else:
            file = ''
#        file = '/Users/shiraiwa/piscope_src/example/images/cmod_logo_official_better_color.eps'

        self.setvar('org_epsfile', file)
        if 'src' not in kywds:
            kywds = self.getvar("kywds")
            kywds['xy'] = self.getvar('xy')
        super(FigEPS, self).__init__(**kywds)

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'eps.png')
        return [idx1]

    @classmethod
    def property_in_file(self):
        return []

    @classmethod
    def get_namebase(self):
        return 'eps'

    @classmethod
    def property_in_palette(self):
        return ["eps"], [["epsscale", "anchor", "alpha_2"]]

#    def set_parent(self, parent):
#        super(FigEPS, self).set_parent(parent)

    def generate_artist(self):

        if self.getvar('epsfile') is None:
            self.import_file()

        container = self.get_figpage()._artists[0]
#        lp=self.getp("loaded_property")
        super(FigEPS, self).generate_artist()

        self.set_frameart(self.getp('frameart'))

    def make_newartist(self):
        self.check_loaded_gp_data()
        if self._image_size == (-1, -1):
            dx = self._eps_bbox[2] - self._eps_bbox[0]
            dy = self._eps_bbox[3] - self._eps_bbox[1]
            x1d, y1d = self.get_gp(0).get_device_point()
            self.get_gp(1).set_device_point(x1d+dx, y1d+dy)
            self._image_scale_str = ('100', '100')

        try:
            self.call_convert()
        except:
            traceback.print_exc()
            a = super(FigEPS, self).make_newartist()
            a.set_alpha(0)
            return a

        a = super(FigEPS, self).make_newartist()
        a.set_alpha(0)

        x1d, y1d = self.get_gp(0).get_device_point()
        if self._image is not None:
            container = self.get_figpage()._artists[0]
            b = create_figimagev(container, self._image, x1d,
                                 y1d, alpha=self._image_alpha)
            self._image_artists = [b]
            b._image_scale = self._image_scale
            b._figobj = weakref.ref(self)
            b._pdf_file = self.getvar('epsfile')[:-3]+'pdf'
            b._eps_file = self.getvar('epsfile')
            b._eps_bbox = self._eps_bbox
            b._is_frameart = self.getp('frameart')

        z = self.getp('zorder')
        for im in self._image_artists:
            im.set_zorder(z)
        return a

    def del_artist(self, artist=None, delall=None):
        for a in self._image_artists:
            #             a.figure.images.remove(a)
            a.figure.artists.remove(a)
        self._image_artists = []

        super(FigEPS, self).del_artist(artist=artist, delall=delall)

    def set_zorder(self, z1, a=None):
        super(FigEPS, self).set_zorder(z1, a)
        z = self.getp('zorder')
        for im in self._image_artists:
            im.set_zorder(z)

    def import_file(self):
        file = self.getvar('org_epsfile')
        self.mk_owndir()
        wdir = self.owndir()
        bname = os.path.basename(file)
        shutil.copyfile(file, os.path.join(wdir, bname))
        self.setvar('epsfile', bname)
        self._eps_bbox = self.get_eps_bbox()

        pname = bname[:-3]+'pdf'
        app = self.get_root_parent().app
        ps2pdf = app.helper.setting['ps2pdf']
        o = sp.Popen([ps2pdf, "-dEPSCrop",
                      os.path.join(wdir, bname),
                      os.path.join(wdir, pname)],
                     stdout=sp.PIPE, stderr=sp.STDOUT).communicate()[0]
        print(o)

    def scale_artist(self, scale, action=True):
        h = super(FigEPS, self).scale_artist(scale, action=action)
        if action:
            pass
        else:
            pass
        return h

    def set_alpha(self, value, a):
        self._image_alpha = value
        for a in self._image_artists:
            a.set_alpha(value)

    def get_alpha(self, a):
        return self._image_alpha

    def set_anchor(self, value, a):
        self._anchor_mode = value

    def get_anchor(self, value):
        return self._anchor_mode

    def set_epsscale(self, value, a):

        self._keep_aspect = value[0]
        self._resize_mode = value[1]
        self._image_scale_str = (value[2], value[3])

        ox = self._eps_bbox[2] - self._eps_bbox[0]
        oy = self._eps_bbox[3] - self._eps_bbox[1]

        dx = int(ox*float(value[2])/100.)
        dy = int(oy*float(value[3])/100.)
        x1d, y1d = self.get_gp(0).get_device_point()
        self.get_gp(1).set_device_point(x1d+dx, y1d+dy)
        self.refresh_artist()

    def get_epsscale(self, a):
        ox = self._eps_bbox[2] - self._eps_bbox[0]
        oy = self._eps_bbox[3] - self._eps_bbox[1]
        ix = self._image.shape[1]
        iy = self._image.shape[0]

        dx = int(ox*float(self._image_scale_str[0])/100.)
        dy = int(oy*float(self._image_scale_str[1])/100.)
        if (ix != dx or iy != dy):
            return (self._keep_aspect,
                    self._resize_mode,
                    float(ix)/ox*100.,
                    float(iy)/oy*100.,)
        else:
            return (self._keep_aspect, self._resize_mode,
                    self._image_scale_str[0],
                    self._image_scale_str[1])

    def call_convert(self):
        x1d, y1d = self.get_gp(0).get_device_point()
        x2d, y2d = self.get_gp(1).get_device_point()
        ox = self._eps_bbox[2] - self._eps_bbox[0]
        oy = self._eps_bbox[3] - self._eps_bbox[1]

        if not self._resize_mode:
            '''
            this mode use geometric mean 
            '''
            r = (float(self._image_scale_str[1]) /
                 float(self._image_scale_str[0]))

            new_size1 = (abs(x1d - x2d), abs(x1d - x2d)*r*oy/ox)
            new_size2 = (abs(y1d - y2d)*ox/oy/r, abs(y1d - y2d))
            new_size = ((new_size1[0]*new_size2[0])**0.5,
                        (new_size1[1]*new_size2[1])**0.5)
            x1d, x2d = calc_newpos_from_anchor(
                x1d, x2d, new_size[0], self._anchor_mode[0])
            y1d, y2d = calc_newpos_from_anchor(
                y1d, y2d, new_size[1], self._anchor_mode[1])
            self.get_gp(0).set_device_point(x1d, y1d)
            self.get_gp(1).set_device_point(x2d, y2d)
        else:
            new_size = (abs(x1d - x2d), abs(y1d-y2d))

        new_size = [int(x) for x in new_size]
        if (new_size[0] == self._image_size[0] and
            new_size[1] == self._image_size[1] and
                self._image is not None):
            return

        wdir = self.owndir()
        des = os.path.join(wdir, 'tmp.png')
        src = os.path.join(wdir, self.getvar('epsfile'))

        params = []
        params += ['-resize', str(int(new_size[0])) +
                   'x'+str(int(new_size[1]))+'!']
        # print 'calling convert',  params

        app = self.get_root_parent().app
        convert = app.helper.setting['convert']
        sp.check_call([convert] + params + [src, des])
        self._image = mpimage.imread(des)
        self._image_size = new_size
        self._image_scale = (new_size[0]/ox, new_size[1]/oy)

    def get_eps_bbox(self, src=None):
        if src is None:
            wdir = self.owndir()
            src = os.path.join(wdir, self.getvar('epsfile'))

        app = self.get_root_parent().app
        gs = app.helper.setting['gs']
        o = sp.Popen(["gs", "-dNOPAUSE", "-dBATCH",
                      "-q", "-sDEVICE=bbox", src], stdout=sp.PIPE, stderr=sp.STDOUT).communicate()[0]
        arr = o.split(' ')
        arr[4] = arr[4].split('\n')[0]
        return float(arr[1]), float(arr[2]), float(arr[3]), float(arr[4])

    def save_data2(self, data):
        names = ('_image_size',
                 '_eps_bbox',
                 '_image_alpha',
                 '_image_scale_str',
                 '_keep_aspect',
                 '_anchor_mode',
                 '_resize_mode')

        var = {name: getattr(self, name) for name in names}
        data['FigEPS'] = (1, var)
        data = super(FigEPS, self).save_data2(data)
        return data

    def load_data2(self, data):
        d = data['FigEPS']

        super(FigEPS, self).load_data2(data)
        for key in d[1]:
            setattr(self, key, d[1][key])

    def get_artists_for_frameart(self):
        '''
        for FigEPS
          _artists is box for picker
          _image_aritsts is actual art

          both should be labeled as frameart
        '''
        return self._image_artists+self._artists



'''

   FigMDSBook

   this class supports "save without data"

   FigMDSBook is made on-the-fly when book is
   opend by MDSScope. 

   When Book is closed it is conveted back to
   normal FigBook, this means if book is closed,
   piscope will save figure data. 
   For book to be saved w/o data, MDSScope should
   keep the book opened.

'''
from ifigure.mto.fig_book import FigBook
from ifigure.mdsplus.fig_mds import FigMds
from ifigure.mto.figobj_param import FigobjParam, FigobjData
from ifigure.utils.setting_parser import iFigureSettingParser as SettingParser

import wx


def convert2figmdsbook(book):
    book.__class__ = FigMDSBook
    if not hasattr(book, '_parallel_page'):
        book._parallel_page = True
    if not hasattr(book, '_parallel_shot'):
        book._parallel_shot = True
    if not hasattr(book, '_parallel_sec'):
        book._parallel_sec = True
    if not hasattr(book, '_use_global_tdi'):
        book._use_global_tdi = False
    if not hasattr(book, '_worker'):
        book._worker = ''


def convert2figbook(book):
    book.__class__ = FigBook


def prepare_dwglobal(book):
    '''
    set global parameters 
    '''
    from ifigure.mdsplus.mdsscope import n_global
    names = ('experiment', 'default_node', 'x', 'y', 'z', 'xerr',
             'yerr', 'title', 'event')
    values = ('', '', '', '', '', '', '', '', '')
    names2 = ('shot', 'global_defaults', 'xmax', 'ymax',
              'xmin', 'ymin', 'x.grid_lines', 'y.grid_lines',
              'global_choice', 'color_order',
              'mpanalysis', 'shot_global', 'use_shot_global',
              'event')
    values2 = ('-1', 0, None, None, None, None, 7, 7,
               [1]*16, ['black', 'red', 'blue', 'g', 'yellow'],
               False, '', False, '')
    gsettings = {n: [v]*n_global for n, v in zip(names, values)}
    gsettings['_flag'] = [[], [], [], []]
    if not book.has_child('dwglobal'):
        #            param = FigobjParam()
        param = FigobjData()
#            param = FigMdsData()
        book.add_child('dwglobal', param)
        param.setvar('globalsets', gsettings)
    elif isinstance(book.dwglobal, FigobjParam):
        var = book.dwglobal.getvar().copy()
        book.dwglobal.destroy()
        param = FigobjData()
        book.add_child('dwglobal', param)
        param.setvar(var)

    for n, v in zip(names2, values2):
        if not book.dwglobal.hasvar(n):
            book.dwglobal.setvar(n, v)
    gs = book.dwglobal.getvar('globalsets')
    if not '_flag' in gs:
        gs['_flag'] = [[], [], [], [], ]

    return book.dwglobal


def convert2figmdsbook2(book):
    #            for page in self.book.walk_page():
    prepare_dwglobal(book)
    if not book.hasvar('mdsplus_server'):
        p = SettingParser()
        v = p.read_setting('mdsplus.default_mdsserver')
        book.setvar('mdsplus_server',
                    v['server'])
#                                 'direct::CMOD')
    if not book.hasvar('mdsscript_main'):
        book.setvar('mdsscript_main', True)
    if not book.hasvar('mdsscope_nticks'):
        book.setvar('mdsscope_nticks', [10, 10])
    if not book.hasvar('mdsscope_autogrid'):
        book.setvar('mdsscope_autogrid', True)
    if not book.hasvar('mdsscope_listenevent'):
        book.setvar('mdsscope_listenevent', True)
    if not book.hasvar('dwscope_filename'):
        book.setvar('dwscope_filename', '')
    if not book.hasvar('global_tdi'):
        book.setvar('global_tdi', '')
    if not book.hasvar('global_tdi_event'):
        book.setvar('global_tdi_event', '')
    convert2figmdsbook(book)
    return book


def new_figmdsbook():
    book = FigBook()
    i_page = book.add_page()
    page = book.get_page(i_page)
    page.add_axes()
    page.realize()
    page.set_area([[0, 0, 1, 1]])

    return convert2figmdsbook2(book)


def add_scopebook(parent, name):
    book = new_figmdsbook()
    parent.add_child(name, book)
    return book


class FigMDSBook(FigBook):
    def add_commonvar(self, vars,  experiment='',
                      default_node='',
                      name='common_vars'):
        '''
        vars = (('psirz', '\\ANALYSIS::TOP:EFIT.RESULTS.G_EQDSK:PSIRZ'), 
         ('rgrid', 'dim_of(\\ANALYSIS::TOP:EFIT.RESULTS.G_EQDSK:PSIRZ)'), 
         ('zgrid', 'dim_of(\\ANALYSIS::TOP:EFIT.RESULTS.G_EQDSK:PSIRZ, 1)'), 
         ('efit_time', 'dim_of(\\ANALYSIS::TOP:EFIT.RESULTS.G_EQDSK:PSIRZ, 2)'), 
         ('fpol', '\\ANALYSIS::TOP:EFIT.RESULTS.G_EQDSK:FPOL'))
        '''
        from ifigure.mdsplus.fig_mdsdata import FigMdsData
        from collections import OrderedDict

        names = self.dwglobal.getvar('shot_global')
        a = [x for x in names.split(',') if len(x.strip()) != 0]
        self.dwglobal.setvar('shot_global', ','.join(a+name.split(',')))
        self.dwglobal.setvar('use_shot_global', True)
        vars = OrderedDict(vars)
        self.add_child(name, FigMdsData())
        cc = self.get_child(name=name)
        cc.setvar('mdsvars', vars)
        cc.setvar('experiment', experiment)
        cc.setvar('default_node', default_node)

    def save2(self, fid=None, olist=None):
        d = self.set_compact_savemode(1)
        olist = FigBook.save2(self, fid=fid, olist=olist)
        self.set_compact_savemode(0, d)
        return olist

    def save_data2(self, data):
        # the first element is version code
        from ifigure.mdsplus.mdsscope import MDSScope
        app = wx.GetApp().TopWindow
        viewer = app.find_bookviewer(self)
        if isinstance(viewer, MDSScope):
            viewer.set_book_scope_param()
        keys = ('_scope_param', '_parallel_page', '_parallel_shot', '_parallel_sec',
                '_use_global_tdi', '_worker')
        param = {key: getattr(self,  key) for key in
                 keys if hasattr(self, key)}
        data['FigMdsBook'] = (1, {}, param)
#        print param
        data = super(FigMDSBook, self).save_data2(data)
        return data

    def load_data2(self, data):
        if 'FigMdsBook' in data:
            v = data['FigMdsBook'][2]
            # print v
            for key in v:
                setattr(self, key, v[key])
        super(FigMDSBook, self).load_data2(data)

    def set_open(self, value):
        ''' 
        make sure to convert to figbook.
        also all loaded data are lost when it is closed.
        '''
        if value is False:
            self.set_compact_savemode(1)
            convert2figbook(self)
        FigBook.set_open(self, value)

    def set_compact_savemode(self, mode, ret=None):
        if mode == 1:
            ret = {}
            for obj in self.walk_tree():
                if isinstance(obj.get_parent(), FigMds):
                    #                    print obj.get_full_path()
                    ret[obj] = obj.prepare_compact_savemode()
                    obj._save_mode = mode
            return ret
        else:
            for obj in self.walk_tree():
                if obj in ret:
                    obj._save_mode = mode
                    obj._var = ret[obj]



from __future__ import print_function
from ifigure.mto.fig_obj import FigObj, mask_negative
from ifigure.widgets.canvas.file_structure import *
from ifigure.mto.axis_user import XUser, YUser
from ifigure.mto.fig_obj import set_mpl_all, get_mpl_first
import ifigure
import os
from ifigure.utils.cbook import nd_iter, LoadImageFile
from ifigure.utils.geom import transform_point
from ifigure.widgets.undo_redo_history import GlobalHistory
from ifigure.widgets.undo_redo_history import UndoRedoArtistProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjProperty
from ifigure.widgets.undo_redo_history import UndoRedoFigobjMethod


import numpy as np
import weakref

from ifigure.utils.args_parser import ArgsParser
import ifigure.utils.debug as debug
dprint1, dprint2, dprint3 = debug.init_dprints('FigAxline')


class FigAxline(FigObj, XUser, YUser):
    def __new__(cls, *args, **kywds):
        """
        FigAxline([x1,x2,x3...], [y1,y2,...])
        x is for vertical line
        y is for horizontal line
        """
        def set_hidden_vars(obj):
            obj._objs = []  # for debug....
            obj._drag_backup = None
            obj._drag_delta = None
            obj._data_extent = None
            return obj

        if 'src' in kywds:
            obj = FigObj.__new__(cls, *args, **kywds)
            obj = set_hidden_vars(obj)
            return obj

        p = ArgsParser()
        p.add_opt('x', [], ['numbers|nonstr', 'dynamic', 'real', 'empty'])
        p.add_opt('y', [], ['numbers|nonstr', 'dynamic', 'real', 'empty'])
        p.set_ndconvert("x", "y")
        p.add_key('draggable', False)

        v, kywds, d, flag = p.process(*args, **kywds)
        if isinstance(v["x"], float) or isinstance(v["x"], int):
            v["x"] = [v["x"]]
        if isinstance(v["y"], float) or isinstance(v["y"], int):
            v["y"] = [v["y"]]

        if not flag:
            raise ValueError('Failed when processing argument')

        obj = FigObj.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)
        for name in v:
            obj.setvar(name, v[name])
        obj.setvar("kywds", kywds)

        return obj

    def __init__(self, *args,  **kywds):
        """
        FigAxline([x]) or FigAxline(x)   
        FigAxline([], [y]) or FigAxline(y)
        """
        XUser.__init__(self)
        YUser.__init__(self)

        args = []
        if 'src' not in kywds:
            kywds = self.getvar("kywds")
        super(FigAxline, self).__init__(*args, **kywds)

    @classmethod
    def isFigAxline(self):
        return True

    @classmethod
    def can_have_child(self, child=None):
        return False

    @classmethod
    def get_namebase(self):
        return 'axline'

    @classmethod
    def property_in_file(self):
        return ["color", "linestyle",
                "linewidth", "marker", "alpha"]

    @classmethod
    def property_in_palette(self):
        return (["line", "marker"],
                [["color_2", "linestyle_2", "linewidth_2",
                  "alpha_2"],
                 ["markerfacecolor_2", "markeredgecolor_2", "marker_2",
                  "markersize_2", "markeredgewidth_2"], ])

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = LoadImageFile(path, 'plot1.png')
        return [idx1]

    def isDraggable(self):
        return self.getvar("draggable")

    def set_parent(self, parent):
        XUser.unset_ax(self)
        YUser.unset_ay(self)
        super(FigAxline, self).set_parent(parent)
        XUser.get_xaxisparam(self)
        YUser.get_yaxisparam(self)

    def args2var(self):
        names0 = self.attr_in_file()
        names = ["x", "y"]
        use_np = [False]*2
        names = names + names0
        use_np = use_np + [False]*len(names0)
        values = self.put_args2var(names, use_np)
        x = values[0]
        y = values[1]
        if x is None:
            return False
        if y is None:
            return False

        return True

    def generate_artist(self):
        # this method generate artist
        # if artist does exist, update artist
        # based on the information specifically
        # managed by fig_obj tree. Any property
        # internally managed by matplotlib
        # does not change
        container = self.get_container()

        if self.isempty() is False:
            return

        x, y = self._eval_xy()
        for x1 in x:
            self._artists.append(container.axvline(x1,
                                                   **(self._var["kywds"])))
        for y1 in y:
            self._artists.append(container.axhline(y1,
                                                   **(self._var["kywds"])))
        self.set_rasterized()
        lp = self.getp("loaded_property")
        if lp is not None:
            for i in range(0, len(lp)):
                if len(self._artists) > i:
                    self.set_artist_property(self._artists[i], lp[i])
            self.delp("loaded_property")

        dprint3("generating axline artist")
        for artist in self._artists:
            artist.figobj = self
            artist.figobj_hl = []
            artist.set_zorder(self.getp('zorder'))

    def del_artist(self, artist=None, delall=False):
        #        if delall:
        artistlist = self._artists
#        else:
#           artistlist=artist
        self.store_loaded_property()

        if len(artistlist) != 0:
            self.highlight_artist(False, artistlist)
            container = self.get_container()
#            container=self._parent._artists[0]
            for a in artistlist:
                a.remove()

        super(FigAxline, self).del_artist(artist=artistlist)

    def highlight_artist(self, val, artist=None):
        if artist is None:
            alist = self._artists
        else:
            alist = artist
        container = self.get_container()
#        container=self._parent._artists[0]
        if val == True:
            hl = []
            x, y = self._eval_xy()

            for x1 in nd_iter(x):
                hl.append(container.axvline(x1,  marker='s',
                                            color='k', linestyle='None',
                                            markeredgewidth=0.5,
                                            markerfacecolor='None'))
            for y1 in nd_iter(y):
                hl.append(container.axhline(y1,  marker='s',
                                            color='k', linestyle='None',
                                            markeredgewidth=0.5,
                                            markerfacecolor='None'))
            for a in alist:
                a.figobj_hl += hl
        else:
            for a in self._artists:
                for hl in a.figobj_hl:
                    #if hl in container.lines:
                    #    container.lines.remove(hl)
                    hl.remove()
                a.figobj_hl = []
#
#   HitTest
#

    def picker_a(self, a, evt):
        hit = False
        axes = a.axes
        if axes is None:
            return False, {}
        x = a.get_xdata()[0]
        xd, yd = transform_point(axes.transData,
                                 x, 0)
        if abs(evt.x - xd) < 5:
            hit = True
        y = a.get_ydata()[0]
        xd, yd = transform_point(axes.transData,
                                 0, y)

        if abs(evt.y - yd) < 5:
            hit = True
        if hit:
            return True, {'child_artist': a}
        return False, {}

    def picker_a0(self, artist, evt):
        hit, extra = self.picker_a(artist, evt)
        if hit:
            return hit, extra, 'area', 3
        else:
            return False, {}, None, 0

    def dragstart(self, a, evt):
        self._drag_backup = (evt.xdata, evt.ydata, evt.x, evt.y)
        self._drag_delta = (0, 0)
        return 0

    def dragstart_a(self, a, evt):
        return self.dragstart(a, evt)

    def drag_a(self, a, evt, shift=None, scale=None):
        return self.drag(a, evt), scale

    def drag(self, a, evt):
        if evt.inaxes is None:
            return 0
        if evt.xdata is None:
            return

        axes = a.axes
        x, y = self._eval_xy()
        dxd = evt.x - self._drag_backup[2]
        dyd = evt.y - self._drag_backup[3]

        if (evt.guiEvent.ShiftDown()):
            if abs(dxd) > 2*abs(dyd):
                dyd = 0
            elif abs(dyd) > 2*abs(dxd):
                dxd = 0
        i = 0
        for x0 in nd_iter(x):
            xd, void = transform_point(axes.transData,
                                       x0, evt.ydata)
            x1, void = transform_point(axes.transData.inverted(),
                                       xd+dxd, void)
            self._artists[i].set_xdata([x1, x1])
            i = i+1
        for y0 in nd_iter(y):
            void, yd = transform_point(axes.transData,
                                       evt.xdata, y0)
            void, y1 = transform_point(axes.transData.inverted(),
                                       void, yd+dyd)

            self._artists[i].set_ydata([y1, y1])
            i = i+1
        self._drag_delta = (dxd, dyd)
        return 1

    def drag_a_get_hl(self, a):
        return self.drag_get_hl(a)

    def drag_get_hl(self, a):
        self._alpha_backup = a.get_alpha()
        for a in self._artists:
            a.set_alpha(0.5)
        return self._artists

    def drag_a_rm_hl(self, a):
        self.drag_rm_hl(a)

    def drag_rm_hl(self, a):
        a.set_alpha(self._alpha_backup)
        for a in self._artists:
            a.set_alpha(self._alpha_backup)

    def dragdone_a(self, a, evt, shift=None, scale=None):
        return self.dragdone(a, evt), scale

    def dragdone(self, a, evt):
        axes = a.axes
        x, y = self._eval_xy()
        dxd, dyd = self._drag_delta

        if (evt.xdata is not None and
                evt.ydata is not None):
            i = 0
            xdata = []
            ydata = []
            for x0 in nd_iter(x):
                xd, void = transform_point(axes.transData,
                                           x0, evt.ydata)
                x1, void = transform_point(axes.transData.inverted(),
                                           xd+dxd, void)
                xdata.append(x1)
                i = i+1
            for y0 in nd_iter(y):
                void, yd = transform_point(axes.transData,
                                           evt.xdata, y0)
                void, y1 = transform_point(axes.transData.inverted(),
                                           void, yd+dyd)
                ydata.append(y1)

                i = i+1
        else:
            xdata = x
            ydata = y

        action = UndoRedoFigobjMethod(self._artists[0],
                                      'data',
                                      (np.array(xdata), np.array(ydata)))
        window = evt.guiEvent_memory.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry([action])
        return 1

    def set_data(self, value, a):
        self.setp('x', value[0])
        self.setp('y', value[1])
        self._data_extent = None
        i = 0
        for x0 in value[0]:
            self._artists[i].set_xdata([x0, x0])
            i = i+1
        for y0 in value[1]:
            self._artists[i].set_ydata([y0, y0])
            i = i+1

    def get_data(self, a=None):
        return (self.getp('x').copy(), self.getp('y').copy())

    def onExport(self, event):
        from matplotlib.artist import getp

        shell = self.get_root_parent().app.shell
        canvas = event.GetEventObject()
        sel = [a() for a in canvas.selection]
        for a in self._artists:
            if a in sel:
                print("Exporting Data to Shell")
                fig_val = {"xdata": getp(a, "xdata"),
                           "ydata": getp(a, "ydata")}
                self.write2shell(fig_val, "fig_val")
                break
        shell.redirectStdout(True)
        text = '#Exporting data as fig_val[\'xdata\'], fig_val[\'ydata\']\"'
        shell.writeOut(text)
        shell.redirectStdout(False)
        # print "exporting data to file"
#
#   setter/getter
#

    def set_alpha(self, value, a):
        set_mpl_all(self._artists, 'alpha', value)

    def get_alpha(self, a):
        return get_mpl_first(self._artists, 'alpha')

    def set_color(self, value, a):
        set_mpl_all(self._artists, 'color', value)

    def get_color(self, a):
        return get_mpl_first(self._artists, 'color')

    def set_linestyle(self, value, a):
        set_mpl_all(self._artists, 'linestyle', value)

    def get_linestyle(self, a):
        return get_mpl_first(self._artists, 'linestyle')

    def set_linewidth(self, value, a):
        set_mpl_all(self._artists, 'linewidth', value)

    def get_linewidth(self, a):
        return get_mpl_first(self._artists, 'linewidth')

    def set_marker(self, value, a):
        set_mpl_all(self._artists, 'marker', value)

    def get_marker(self, a):
        return get_mpl_first(self._artists, 'marker')

    def set_markerfacecolor(self, value, a):
        set_mpl_all(self._artists, 'markerfacecolor', value)

    def get_markerfacecolor(self, a):
        return get_mpl_first(self._artists, 'markerfacecolor')

    def set_markeredgecolor(self, value, a):
        set_mpl_all(self._artists, 'markeredgecolor', value)

    def get_markeredgecolor(self,  a):
        return get_mpl_first(self._artists, 'markeredgecolor')

    def set_markersize(self, value, a):
        set_mpl_all(self._artists, 'markersize', value)

    def get_markersize(self, a):
        return get_mpl_first(self._artists, 'markersize')

    def set_markeredgewidth(self, value, a):
        set_mpl_all(self._artists, 'markeredgewidth', value)

    def get_markeredgewidth(self,  a):
        return get_mpl_first(self._artists, 'markeredgewidth')

#
#   data extent
#
    def get_data_extent(self):
        if self._data_extent is not None:
            return self._data_extent
        x, y = self._eval_xy()
        d = [None]*4
        if len(x) != 0:
            d[0] = min(x)
            d[1] = max(x)
        if len(y) != 0:
            d[2] = min(y)
            d[3] = max(y)
        self._data_extent = d
        return self._data_extent

#
#   range setting
#
    def get_xrange(self, xrange=[None, None], scale='linear'):
        x, y = self._eval_xy()
        if scale == 'log':
            x = mask_negative(x)
        if x.size > 0:
            return self._update_range(xrange, [np.nanmin(x), np.nanmax(x)])
        else:
            return super(FigAxline, self).get_xrange(xrange, scale=scale)

    def get_yrange(self, yrange=[None, None],
                   xrange=[None, None], scale='linear'):
        x, y = self._eval_xy()
        if scale == 'log':
            y = mask_negative(y)
        if y.size != 0:
            return self._update_range(yrange, [np.nanmin(y), np.nanmax(y)])
        else:
            return super(FigAxline, self).get_yrange(yrange=yrange,
                                                     xrange=xrange, scale=scale)

    def save_data2(self, data=None):
        def check(obj, name):
            return obj.getp(name) is obj.getvar(name)
        if data is None:
            data = {}
        var = {'x': check(self, 'x'),
               'y': check(self, 'y')}

        if var["x"] is not True:
            var["xdata"] = self.getp("x")
        if var["y"] is not True:
            var["ydata"] = self.getp("y")

        #dprint1('save_data2', var)
        data['FigAxline'] = (1, var)
        data = super(FigAxline, self).save_data2(data)
        return data

    def load_data2(self, data):
        d = data['FigAxline']
        super(FigAxline, self).load_data2(data)
        var = d[1]
        names = ["x", "y"]
        for name in names:
            if var[name]:
                self.setp(name,
                          self.getvar(name))
            else:
                self.setp(name, var[name+'data'])

    def _eval_xy(self):
        if self.getp('use_var'):
            success = self.handle_use_var()
            if not success:
                return None, None
        return self.getp(("x", "y"))



import wx

# ----------------------------------------------------------------------


class ShowImageFrame(wx.Frame):
    def __init__(self, parent=None, bitmap=None):
        wx.Frame.__init__(self, parent, -1)
        self.bitmap = bitmap
        self.SetSizer(wx.BoxSizer(wx.HORIZONTAL))
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Layout()
        self.Show()

        bmp_obj = wx.BitmapDataObject()
        bmp_obj.SetBitmap(self.bitmap)
        wx.TheClipboard.Open()
        wx.TheClipboard.SetData(bmp_obj)
        wx.TheClipboard.Close()
        wx.TheClipboard.Flush()

    def OnPaint(self, evt):
        dc = wx.PaintDC(self)
        dc.BeginDrawing()
        dc.Clear()
        dc.DrawBitmap(self.bitmap,  0, 0, True)
        dc.EndDrawing()



from __future__ import print_function
#####################################
#
# debug.py
#
#    provide a simple interface to make debug prints
#    allow for controling turn on and off of debug
#    print for each module
#
# Usage:
#    (in import section)
#    import ifigure.utils.debug as debug
#    dprint1, dprint2, dprint3 = debug.init_dprints('ArgsParser', level=0)
#
#    (then use it as follows)
#    debug.set_level('ArgsParser', 1)  # set level for ArsgParser 1
#    dprint1('hogehogehoge')           # print something
#
#    level 1 (dprint1) : usr feedback which will be turn on normally
#    level 2 (dprint2) : first level of debug print
#    level 3 (dprint3) : second level of debug print
#    setting debug_default_level to 0 will turn off all error print
#    (silent mode)


import sys
import traceback
from ifigure.widgets.redirect_output import RedirectOutput

debug_mode = 1
debug_modes = {}
debug_default_level = 1

debug_stdout = RedirectOutput(sys.stdout, sys.stderr)


def dprint(*args):
    s = ''
    for item in args:
        s = s + ' ' + str(item)
    if debug_mode != 0:
        import sys
        ostdout = sys.stdout
        sys.stdout = debug_stdout
        print('DEBUG('+str(debug_mode)+')::'+s)
        sys.stdout = ostdout


def find_by_id(_id_):
    '''
    find an object using id 
    '''
    import gc
    for obj in gc.get_objects():
        if id(obj) == _id_:
            return obj
    raise Exception("No found")


class DPrint(object):
    def __init__(self, name, level):
        self.name = name
        self.level = level

    def __call__(self, *args, **kargs):
        if 'stack' in kargs:
            traceback.print_stack()
        s = ''
        for item in args:
            s = s + str(item)
        if debug_modes[self.name] >= self.level:
            import sys
            ostdout = sys.stdout
            sys.stdout = debug_stdout
            print('DEBUG('+str(self.name)+')::'+s)
            sys.stdout = ostdout


def prints(n):
    return DPrint(n, 1), DPrint(n, 2), DPrint(n, 3)


def set_level(name, level):
    debug_modes[name] = level


def init_dprints(name, level=debug_default_level):
    set_level(name, level)
    return prints(name)




from __future__ import print_function
import subprocess as sp
import os
import Pyro4


class ProxyEfit(object):
    def filename(self, shot, time, prefix='k'):
        # time in [s]
        a = '0'*5
        b = str(int(time*1000))
        c = a[:-len(b)]+b
        return '.'.join([prefix+str(int(shot)), c])

    def run_minus_5(self, shot=1120710020,
                    time=1,
                    tree='ANALYSIS',
                    exe='/usr/local/cmod/codes/efit/bin/efitd6565d'):
        # time in [s]
        p = sp.Popen(exe, stdin=sp.PIPE)
        p.stdin.write('-5\n')
        p.stdin.write('0\n')
        p.stdin.write('0\n')
        p.stdin.write(str(shot)+', '+str(int(time*1000))+', ' +
                      str(int(time*1000))+', ' + '1\n')

        p.stdin.write(tree+'\n')
        p.stdin.write("\\" + tree + "::\n")
        print(p.wait())

        path = self.filename(shot, time)
        content = ''
        if os.path.exists(path):
            f = open(path, 'r')
            content = f.read()
            f.close()
            os.remove(path)
        return content

    def run_2(self, shot=1120710020, time=1, namelist='',
              prefix='m',
              exe='/usr/local/cmod/codes/efit/bin/efitd6565d'):
        # time in [s]
        # save content as file
        kpath = self.filename(shot, time, prefix='k')
        f = open(kpath, 'w')
        f.write(namelist)
        f.close()

        # run mode 2 of efit
        p = sp.Popen(exe, stdin=sp.PIPE)
        p.stdin.write('2\n')
        p.stdin.write('1\n')
        p.stdin.write(kpath+'\n')
        p.wait()
        # read g and a file
        apath = self.filename(shot, time, prefix='a')
        gpath = self.filename(shot, time, prefix='g')
        acontent = ''
        if os.path.exists(apath):
            f = open(apath, 'r')
            acontent = f.read()
            f.close()
            os.remove(apath)
        gcontent = ''
        if os.path.exists(gpath):
            f = open(gpath, 'r')
            gcontent = f.read()
            f.close()
            os.remove(gpath)

        os.remove(kpath)
        return acontent, gcontent


def main():
    p = ProxyEfit()
    nameserver = Pyro4.locateNS()
    Pyro4.Daemon.serveSimple(
        {
            p: "proxy_efit"
        },
        ns=True,
        host=os.getenv('HOSTNAME'))


if __name__ == '__main__':
    # check program
    main()



import sys
import wx
import time
import ifigure.utils.debug as debug
dprint1, dprint2, dprint3 = debug.init_dprints('import_dw')

replace_shot = True


def interpret_globals(com, var, globals):
    globals[com] = var.strip()
    return globals


def interpret_globals_int(com, var, globals):
    globals[com] = int(var)
    return globals


def interpret_plots(com1, com2, var, plots, use_int=False):
    a = com1.split('_')
    c = int(a[2])-1
    r = int(a[1])-1
    if len(plots) <= c:
        return plots
    if len(plots[c]) <= r:
        return plots
    if use_int:
        plots[c][r][com2] = int(var)
    else:
        plots[c][r][com2] = var.strip()
#    print c, r, com2, var
    return plots


def import_dw(file=''):
    if file == '':
        open_dlg = wx.FileDialog(None, message="Select DWscope to open",
                                 wildcard='*.dat', style=wx.FD_OPEN)
        if open_dlg.ShowModal() == wx.ID_OK:
            file = open_dlg.GetPath()
            open_dlg.Destroy()
        else:
            return [None, None, None, None, None]

    f = open(file, 'r')
    lines = f.readlines()
    k = 0
    setting = {}
#    txxx = time.time()
    globals = {}
    geom = [300, 300]
    for line in lines:
        line = line.strip('\r\n')
        a = line.split(':')
        if len(a) < 2:
            continue
        com = a[0]
        var = ':'.join(a[1:])
        vararr = var.split('|||')
        vararr = [s.strip() for s in vararr]
        vararr = [s for s in vararr if s != '']
        var = '\n'.join(vararr)
        coms = com.split('.')

        if (coms[1][0:4] == 'plot' and len(coms) == 3):
            plots = interpret_plots(coms[1], coms[2], var, plots)
        elif (coms[1][0:4] == 'plot' and len(coms) == 4):
            if '.'.join(coms[2:]) == 'x.grid_lines':
                plots = interpret_plots(coms[1], '.'.join(coms[2:]), var,
                                        plots, use_int=True)
            elif '.'.join(coms[2:]) == 'y.grid_lines':
                plots = interpret_plots(coms[1], '.'.join(coms[2:]), var,
                                        plots, use_int=True)
            else:
                pass
        elif (coms[1][0:6] == 'global' and len(coms) == 3):
            globals = interpret_globals(coms[2], var, globals)
        elif (coms[1][0:6] == 'global' and len(coms) == 4):
            if '.'.join(coms[2:]) == 'x.grid_lines':
                globals = interpret_globals_int(
                    '.'.join(coms[2:]), var, globals)
            elif '.'.join(coms[2:]) == 'y.grid_lines':
                globals = interpret_globals_int(
                    '.'.join(coms[2:]), var, globals)
            else:
                pass
        elif coms[1][0:14] == 'rows_in_column':
            c = int(coms[1][15:])-1
            r = int(var)
            plots[c] = [{} for k in range(r)]
        elif coms[1] == 'columns':
            plots = [[] for k in range(int(var))]
        elif coms[1][:8] == 'geometry':
            geom = [int(x) for x in var.split('+')[0].split('x')]
        else:
            setting[coms[-1]] = var
        wx.GetApp().Yield(True)
#       wx.Yield()
        k = k+1
    f.close()

    if replace_shot:
        for x in plots:
            for y in x:
                if 'title' in y:
                    y['title'] = y['title'].replace('$shot', '_shots')
                    y['title'] = y['title'].replace('$SHOT', '_shots')
#    dprint1(time.time() - txxx)
    return setting, globals, plots, file, geom



'''
   Utility class for triplot, tripcolor
'''
import ifigure.events
import numpy as np


class TrianglePlots(object):
    def __init__(self):
        if not hasattr(self, '_tri'):
            self._tri = None

    def get_tri(self):
        return self._tri

    def set_mask(self, mask, a=None):
        self.setp('mask', mask)
        self.set_bmp_update(False)

    def get_mask(self, a=None):
        return self.getp('mask')

    def get_masked_tri(self):
        mask = self.getp('mask')
        if mask is not None:
            tri = self._tri.compress(1 - mask, axis=0)
        else:
            tri = self._tri
        return tri

    def mask_array(self, mask, no_draw=False):
        self.set_mask(mask)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)

    def mask_outside(self, edge, op=np.logical_or, no_draw=False):
        import ifigure.utils.triangulation_wrapper
        x, y = self.getp(('x', 'y'))
        tri = self._tri
        mask = ifigure.utils.triangulation_wrapper.mask_outside(x, y, tri, edge,
                                                                mask=self.getp('mask'), op=op)
        self.set_mask(mask)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)

    def mask_inside(self, edge, op=np.logical_or, no_draw=False):
        import ifigure.utils.triangulation_wrapper
        x, y = self.getp(('x', 'y'))
        tri = self._tri
        mask = ifigure.utils.triangulation_wrapper.mask_inside(x, y, tri, edge,
                                                               mask=self.getp('mask'), op=op)
        self.set_mask(mask)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)

    def mask_none(self, no_draw=False):
        self.set_mask(None)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)

    def mask_skew(self, th, op=np.logical_or, no_draw=False):
        import ifigure.utils.triangulation_wrapper
        x, y = self.getp(('x', 'y'))
        tri = self._tri
        mask = ifigure.utils.triangulation_wrapper.mask_skew(x, y, tri, th,
                                                             mask=self.getp('mask'), op=op)
        self.set_mask(mask)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)

    def get_tri_area(self, th, op=np.logical_or, no_draw=False):
        import ifigure.utils.triangulation_wrapper
        x, y = self.getp(('x', 'y'))
        tri = self._tri
        area = ifigure.utils.triangulation_wrapper.get_area(x, y, tri)
        return area

    def mask_area(self, th, op=np.logical_or, no_draw=False):
        import ifigure.utils.triangulation_wrapper
        x, y = self.getp(('x', 'y'))
        tri = self._tri
        mask = ifigure.utils.triangulation_wrapper.mask_area(x, y, tri, th,
                                                             mask=self.getp('mask'), op=op)
        self.set_mask(mask)

        self.del_artist(delall=True)
        if hasattr(self, "_data_extent"):
            self._data_extent = None
        self.generate_artist()
        if not no_draw:
            ifigure.events.SendPVDrawRequest(self)




import wx
import sys

try:
    from wx import glcanvas
    haveGLCanvas = True
except ImportError:
    haveGLCanvas = False

try:
    # The Python OpenGL package can be found at
    # http://PyOpenGL.sourceforge.net/
    from OpenGL.GL import *
    from OpenGL.GLUT import *
    haveOpenGL = True
except ImportError:
    haveOpenGL = False

# ----------------------------------------------------------------------


buttonDefs = {
    wx.NewIdRef(count=1): ('CubeCanvas',      'Cube'),
    wx.NewIdRef(count=1): ('ConeCanvas',      'Cone'),
}


class ButtonPanel(wx.Panel):
    def __init__(self, parent):
        wx.Panel.__init__(self, parent, -1)

        box = wx.BoxSizer(wx.VERTICAL)
        box.Add((20, 30))
        keys = list(buttonDefs.keys())
        keys.sort()
        for k in keys:
            text = buttonDefs[k][1]
            btn = wx.Button(self, k, text)
            box.Add(btn, 0, wx.ALIGN_CENTER | wx.ALL, 15)
            self.Bind(wx.EVT_BUTTON, self.OnButton, btn)

        # ** Enable this to show putting a GLCanvas on the wx.Panel
        if 1:
            c = CubeCanvas(self)
            c.SetMinSize((200, 200))
            box.Add(c, 0, wx.ALIGN_CENTER | wx.ALL, 15)

        self.SetAutoLayout(True)
        self.SetSizer(box)

    def OnButton(self, evt):
        if not haveGLCanvas:
            dlg = wx.MessageDialog(self,
                                   'The GLCanvas class has not been included with this build of wxPython!',
                                   'Sorry', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()

        elif not haveOpenGL:
            dlg = wx.MessageDialog(self,
                                   'The OpenGL package was not found.  You can get it at\n'
                                   'http://PyOpenGL.sourceforge.net/',
                                   'Sorry', wx.OK | wx.ICON_WARNING)
            dlg.ShowModal()
            dlg.Destroy()

        else:
            canvasClassName = buttonDefs[evt.GetId()][0]
            canvasClass = eval(canvasClassName)
            frame = wx.Frame(None, -1, canvasClassName, size=(400, 400))
            canvas = canvasClass(frame)
            frame.Show(True)


class MyCanvasBase(glcanvas.GLCanvas):
    def __init__(self, parent):
        glcanvas.GLCanvas.__init__(self, parent, -1)
        self.init = False
        self.context = glcanvas.GLContext(self)

        # initial mouse position
        self.lastx = self.x = 30
        self.lasty = self.y = 30
        self.size = None
        self.Bind(wx.EVT_ERASE_BACKGROUND, self.OnEraseBackground)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnMouseDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnMouseUp)
        self.Bind(wx.EVT_MOTION, self.OnMouseMotion)

    def OnEraseBackground(self, event):
        pass  # Do nothing, to avoid flashing on MSW.

    def OnSize(self, event):
        wx.CallAfter(self.DoSetViewport)
        event.Skip()

    def DoSetViewport(self):
        size = self.size = self.GetClientSize()
        self.SetCurrent(self.context)
        glViewport(0, 0, size.width, size.height)

    def OnPaint(self, event):
        dc = wx.PaintDC(self)
        self.SetCurrent(self.context)
        if not self.init:
            self.InitGL()
            self.init = True
        self.OnDraw()

    def OnMouseDown(self, evt):
        self.CaptureMouse()
        self.x, self.y = self.lastx, self.lasty = evt.GetPosition()

    def OnMouseUp(self, evt):
        self.ReleaseMouse()

    def OnMouseMotion(self, evt):
        if evt.Dragging() and evt.LeftIsDown():
            self.lastx, self.lasty = self.x, self.y
            self.x, self.y = evt.GetPosition()
            self.Refresh(False)


class CubeCanvas(MyCanvasBase):
    def InitGL(self):
        # set viewing projection
        glMatrixMode(GL_PROJECTION)
        glFrustum(-0.5, 0.5, -0.5, 0.5, 1.0, 3.0)

        # position viewer
        glMatrixMode(GL_MODELVIEW)
        glTranslatef(0.0, 0.0, -2.0)

        # position object
        glRotatef(self.y, 1.0, 0.0, 0.0)
        glRotatef(self.x, 0.0, 1.0, 0.0)

        glEnable(GL_DEPTH_TEST)
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)

    def OnDraw(self):
        # clear color and depth buffers
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)

        # draw six faces of a cube
        glBegin(GL_QUADS)
        glNormal3f(0.0, 0.0, 1.0)
        glVertex3f(0.5, 0.5, 0.5)
        glVertex3f(-0.5, 0.5, 0.5)
        glVertex3f(-0.5, -0.5, 0.5)
        glVertex3f(0.5, -0.5, 0.5)

        glNormal3f(0.0, 0.0, -1.0)
        glVertex3f(-0.5, -0.5, -0.5)
        glVertex3f(-0.5, 0.5, -0.5)
        glVertex3f(0.5, 0.5, -0.5)
        glVertex3f(0.5, -0.5, -0.5)

        glNormal3f(0.0, 1.0, 0.0)
        glVertex3f(0.5, 0.5, 0.5)
        glVertex3f(0.5, 0.5, -0.5)
        glVertex3f(-0.5, 0.5, -0.5)
        glVertex3f(-0.5, 0.5, 0.5)

        glNormal3f(0.0, -1.0, 0.0)
        glVertex3f(-0.5, -0.5, -0.5)
        glVertex3f(0.5, -0.5, -0.5)
        glVertex3f(0.5, -0.5, 0.5)
        glVertex3f(-0.5, -0.5, 0.5)

        glNormal3f(1.0, 0.0, 0.0)
        glVertex3f(0.5, 0.5, 0.5)
        glVertex3f(0.5, -0.5, 0.5)
        glVertex3f(0.5, -0.5, -0.5)
        glVertex3f(0.5, 0.5, -0.5)

        glNormal3f(-1.0, 0.0, 0.0)
        glVertex3f(-0.5, -0.5, -0.5)
        glVertex3f(-0.5, -0.5, 0.5)
        glVertex3f(-0.5, 0.5, 0.5)
        glVertex3f(-0.5, 0.5, -0.5)
        glEnd()

        if self.size is None:
            self.size = self.GetClientSize()
        w, h = self.size
        w = max(w, 1.0)
        h = max(h, 1.0)
        xScale = 180.0 / w
        yScale = 180.0 / h
        glRotatef((self.y - self.lasty) * yScale, 1.0, 0.0, 0.0)
        glRotatef((self.x - self.lastx) * xScale, 0.0, 1.0, 0.0)

        self.SwapBuffers()


class ConeCanvas(MyCanvasBase):
    def InitGL(self):
        glMatrixMode(GL_PROJECTION)
        # camera frustrum setup
        glFrustum(-0.5, 0.5, -0.5, 0.5, 1.0, 3.0)
        glMaterial(GL_FRONT, GL_AMBIENT, [0.2, 0.2, 0.2, 1.0])
        glMaterial(GL_FRONT, GL_DIFFUSE, [0.8, 0.8, 0.8, 1.0])
        glMaterial(GL_FRONT, GL_SPECULAR, [1.0, 0.0, 1.0, 1.0])
        glMaterial(GL_FRONT, GL_SHININESS, 50.0)
        glLight(GL_LIGHT0, GL_AMBIENT, [0.0, 1.0, 0.0, 1.0])
        glLight(GL_LIGHT0, GL_DIFFUSE, [1.0, 1.0, 1.0, 1.0])
        glLight(GL_LIGHT0, GL_SPECULAR, [1.0, 1.0, 1.0, 1.0])
        glLight(GL_LIGHT0, GL_POSITION, [1.0, 1.0, 1.0, 0.0])
        glLightModelfv(GL_LIGHT_MODEL_AMBIENT, [0.2, 0.2, 0.2, 1.0])
        glEnable(GL_LIGHTING)
        glEnable(GL_LIGHT0)
        glDepthFunc(GL_LESS)
        glEnable(GL_DEPTH_TEST)
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        # position viewer
        glMatrixMode(GL_MODELVIEW)
        # position viewer
        glTranslatef(0.0, 0.0, -2.0)
        #
        glutInit(sys.argv)

    def OnDraw(self):
        # clear color and depth buffers
        glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT)
        # use a fresh transformation matrix
        glPushMatrix()
        # position object
        #glTranslate(0.0, 0.0, -2.0)
        glRotate(30.0, 1.0, 0.0, 0.0)
        glRotate(30.0, 0.0, 1.0, 0.0)

        glTranslate(0, -1, 0)
        glRotate(250, 1, 0, 0)
        glutSolidCone(0.5, 1, 30, 5)
        glPopMatrix()
        glRotatef((self.y - self.lasty), 0.0, 0.0, 1.0)
        glRotatef((self.x - self.lastx), 1.0, 0.0, 0.0)
        # push into visible buffer
        self.SwapBuffers()


# ----------------------------------------------------------------------


def runTest(frame):
    win = ButtonPanel(frame)
    return win


overview = """\
"""


if __name__ == '__main__':
    import sys
    import os
    app = wx.App()
    frame = wx.Frame(None)
    frame.Show()
    runTest(frame)
    app.MainLoop()



'''
   FigPlotC

     A figplot which a user can edit value by mouse
'''
import numpy as np
import wx
import os
import ifigure
from ifigure.mto.fig_plot import FigPlot
from ifigure.utils.geom import transform_point
from ifigure.utils.cbook import ProcessKeywords, LoadImageFile
from ifigure.widgets.undo_redo_history import GlobalHistory, UndoRedoFigobjMethod
from ifigure.utils.edit_list import DialogEditList
from ifigure.mto.fig_control import FigControl


def _copy(x):
    if x is None:
        return
    return x.copy()


class FigPlotC(FigPlot, FigControl):
    def __new__(cls, *args, **kargs):
        draggable, kargs = ProcessKeywords(kargs, 'draggable', True)
        enabled_point, kargs = ProcessKeywords(kargs, 'enabled_point', 'all')
        obj = FigPlot.__new__(cls, *args, **kargs)
        if obj is not None:
            obj.setvar('draggable', draggable)
            if not obj.hasvar('enabled_point'):
                obj.setvar('enabled_point', enabled_point)
        return obj

    def __init__(self, *args, **kargs):
        FigPlot.__init__(self, *args, **kargs)
        FigControl.__init__(self)
        self._figc_hit = -1
        self._enable_a = None
        self._mark_a = None

    @classmethod
    def attr_in_file(self):
        return ['enabled_point'] + FigPlot.attr_in_file()

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = LoadImageFile(path, 'plotc.png')
        return [idx1]

    @classmethod
    def get_namebase(self):
        return 'plotc'

    def args2var(self):
        val = FigPlot.args2var(self)
        if self.getvar('enabled_point') == 'all':
            self.setp('enabled_point', [True]*(self.getp('x').size))
        return val

    def isDraggable(self):
        return self.getvar("draggable")

    def generate_artist(self):
        FigPlot.generate_artist(self)
        container = self.get_container()
        a = self._artists[0]
        x = a.get_xdata()
        y = a.get_ydata()

        vm = self.getp('marked_point')
        if vm is not None:
            if self._mark_a is not None:
                self._mark_a.remove()
                self._mark_a = None

            a3 = container.plot(x[vm].copy(), y[vm].copy(), marker='o',
                                color='k', linestyle='None',
                                markerfacecolor='k',
                                markeredgecolor='k',
                                markersize=6,
                                scalex=False, scaley=False)
            self._mark_a = a3[0]

        idx = np.where([not xx for xx in self.getp('enabled_point')])[0]
        if self._enable_a is not None:
            self._enable_a.remove()
            self._enable_a = None
        if idx.size == 0:
            return

        marker = a.get_marker()
        fc = 'white'
        ec = a.get_markeredgecolor()
        ms = a.get_markersize()
        a2 = container.plot(x[idx].copy(), y[idx].copy(), marker=marker,
                            color='k', linestyle='None',
                            markerfacecolor=fc,
                            markeredgecolor=ec,
                            markersize=ms,
                            scalex=False, scaley=False)

        self._enable_a = a2[0]

    def del_artist(self, artist=None, delall=False):
        if self._enable_a is not None:
            self._enable_a.remove()
            self._enable_a = None
        if self._mark_a is not None:
            self._mark_a.remove()
            self._mark_a = None

        return FigPlot.del_artist(self, artist=artist, delall=delall)

    def canvas_menu(self):
        m = [('Add Point', self.onAddPoint, None), ]
        if self._figc_hit != -1:
            if self.getp('enabled_point')[self._figc_hit]:
                m = m + [('Delete point', self.onDelPoint, None),
                         ('Edit point', self.onEditPoint, None),
                         ('Disable point', self.onDisablePoint, None), ]
            else:
                m = m + [('Delete point', self.onDelPoint, None),
                         ('Edit point', self.onEditPoint, None),
                         ('Enable point', self.onEnablePoint, None), ]
            vm = self.get_mark_data()
            if vm[self._figc_hit]:
                m = m + [('Unmark point', self.onUnmarkPoint, None),
                         ('Reset mark', self.onResetMark, None),
                         ('Export mark', self.onExportMark, None), ]
            else:
                m = m + [('Mark point', self.onMarkPoint, None),
                         ('Reset mark', self.onResetMark, None),
                         ('Export mark', self.onExportMark, None), ]

        return m + FigPlot.canvas_menu(self)

    def onEditPoint(self, evt):
        def add_list(l, name, self, idx):
            value = self.getp(name)
            l.append([name, str(value[idx]), 0, None])

        from ifigure.utils.edit_list import DialogEditList
        l = []
        names = ['x', 'y']
        if self.getp('xerr') is not None:
            names.append('xerr')
        if self.getp('yerr') is not None:
            names.append('yerr')
        for name in names:
            add_list(l, name, self, self._figc_hit)

        window = evt.GetEventObject().GetTopLevelParent()
        value = DialogEditList(l, modal=True,
                               style=wx.DEFAULT_DIALOG_STYLE | wx.RESIZE_BORDER,
                               tip=None,
                               parent=window)
        if value[0]:
            v = value[1]
        else:
            return
        x = _copy(self.getp('x'))
        x[self._figc_hit] = v[names.index('x')]
        y = _copy(self.getp('y'))
        y[self._figc_hit] = v[names.index('y')]
        yerr = _copy(self.getp('yerr'))
        if yerr is not None:
            yerr[self._figc_hit] = v[names.index('yerr')]
        xerr = _copy(self.getp('xerr'))
        if xerr is not None:
            xerr[self._figc_hit] = v[names.index('xerr')]

        actions = [UndoRedoFigobjMethod(self._artists[0],
                                        'data', (x, y)), ]
        if self._mpl_cmd != 'plot':
            actions.append(UndoRedoFigobjMethod(self._artists[0],
                                                'errdata', (xerr, yerr)))
        window = evt.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry(actions)
        return 1

    def onDelPoint(self, evt):
        v = self.getp('enabled_point')[:]
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        del v[self._figc_hit]
        vm = self.getp('marked_point')[:]
        del vm[self._figc_hit]
        x = np.delete(x, self._figc_hit)
        y = np.delete(y, self._figc_hit)
        actions = [UndoRedoFigobjMethod(self._artists[0],
                                        'data', (x, y)), ]
        if self._mpl_cmd != 'plot':
            xerr = _copy(self.getp('xerr'))
            yerr = _copy(self.getp('yerr'))
            if xerr is not None:
                xerr = np.delete(xerr, self._figc_hit)
            if yerr is not None:
                yerr = np.delete(yerr, self._figc_hit)
            actions.append(UndoRedoFigobjMethod(self._artists[0],
                                                'errdata', (xerr, yerr)))
        actions.append(UndoRedoFigobjMethod(self._artists[0],
                                            'enabled_point', v))
        actions.append(UndoRedoFigobjMethod(self._artists[0],
                                            'marked_point', vm))
        window = evt.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry(actions)
        return 1

    def onAddPoint(self, evt):
        axes = self._artists[0].axes
        if axes is None:
            return
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        xed, yed = axes.transData.transform(self._figc_hit_pos)

        for k in range(x.size-1):
            x0 = x[k]
            y0 = y[k]
            x1 = x[k+1]
            y1 = y[k+1]
            x0d, y0d = axes.transData.transform([x0, y0])
            x1d, y1d = axes.transData.transform([x1, y1])
            d = np.sqrt((x0d-x1d)**2 + (y0d-y1d)**2)
            m1 = ((x1d-xed)*(x1d-x0d) + (y1d-yed)*(y1d-y0d))/d/d
            if (m1 < 0 or m1 > 1):
                continue
            m2 = ((x1d-xed)*(y1d-y0d) - (y1d-yed)*(x1d-x0d))/d
            if (m2 > 5 or m2 < -5):
                continue
            break
        else:
            return 1

        v = self.getp('enabled_point')[:]
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        v.insert(k+1, True)
        vm = self.getp('marked_point')[:]
        vm.insert(k+1, True)

        x = np.insert(x, k+1, self._figc_hit_pos[0])
        y = np.insert(y, k+1, self._figc_hit_pos[1])
        actions = [UndoRedoFigobjMethod(self._artists[0],
                                        'data', (x, y)), ]
        if self._mpl_cmd != 'plot':
            xerr = _copy(self.getp('xerr'))
            yerr = _copy(self.getp('yerr'))
            if xerr is not None:
                xerr = np.insert(xerr, k+1, xerr[k])
            if yerr is not None:
                yerr = np.insert(yerr, k+1, yerr[k])
            actions.append(UndoRedoFigobjMethod(self._artists[0],
                                                'errdata', (xerr, yerr)))
        actions.append(UndoRedoFigobjMethod(self._artists[0],
                                            'enabled_point', v))
        actions.append(UndoRedoFigobjMethod(self._artists[0],
                                            'marked_point', vm))

        window = evt.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry(actions)
        return 1

    def onEnablePoint(self, evt):
        v = self.getp('enabled_point')[:]
        v[self._figc_hit] = True
        # x, y is the same, this triggers control_changed_callback
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        actions = [UndoRedoFigobjMethod(self._artists[0],
                                        'data', (x, y)),
                   UndoRedoFigobjMethod(self._artists[0],
                                        'enabled_point', v)]

        window = evt.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry(actions)
        return 1

    def onDisablePoint(self, evt):
        v = self.getp('enabled_point')[:]
        v[self._figc_hit] = False

        # x, y is the same, this triggers control_changed_callback
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        actions = [UndoRedoFigobjMethod(self._artists[0],
                                        'data', (x, y)),
                   UndoRedoFigobjMethod(self._artists[0],
                                        'enabled_point', v)]
        window = evt.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry(actions)
        return 1

    def onMarkPoint(self, evt):
        vm = self.getp('marked_point')
        if vm is None:
            l = len(self.getp('enabled_point'))
            self.setp('marked_point', [False]*l)
            vm = self.getp('marked_point')
        vm[self._figc_hit] = True
        self.setp('marked_point', vm)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)

        import ifigure.events
        w = evt.GetEventObject().GetTopLevelParent()
        w.draw()
        return 1

    def onUnmarkPoint(self, evt):
        vm = self.getp('marked_point')
        if vm is None:
            l = len(self.getp('enabled_point'))
            self.setp('marked_point', [False]*l)
            vm = self.getp('marked_point')
        vm[self._figc_hit] = False
        self.setp('marked_point', vm)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)

        import ifigure.events
        w = evt.GetEventObject().GetTopLevelParent()
        w.draw()
        return 1

    def onResetMark(self, evt):
        vm = self.getp('marked_point')
        if vm is None:
            l = len(self.getp('enabled_point'))
            self.setp('marked_point', [False]*l)
            vm = self.getp('marked_point')
        for x in range(len(vm)):
            vm[x] = False
        self.setp('marked_point', vm)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)
        import ifigure.events
        w = evt.GetEventObject().GetTopLevelParent()
        w.draw()
        return 1

    def onExportMark(self, evt):
        vm = self.getp('marked_point')
        if vm is None:
            return
        a = self._artists[0]
        x = a.get_xdata()
        y = a.get_ydata()
        xx = x[vm]
        yy = y[vm]
        data = {'xdata': xx, 'ydata': yy}
        self._export_shell(data, 'data', '')

    def enable_point(self, idx, value):
        if len(self.getp('enabled_point')) > idx+1:
            return
        self.getp('enabled_point')[self._figc_hit] = value

    def do_update_artist(self):
        if not self.isempty():
            a1 = self._artists[0]
            self.del_artist(delall=True)
        if self.isempty() and not self._suppress:
            self.generate_artist()
            a2 = self._artists[0]
            if a1 != a2:
                ifigure.events.SendReplaceEvent(self, a1=a1, a2=a2)

    def set_enabled_point(self, value, a):
        self.setp('enabled_point', value)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)

    def get_enabled_point(self, a):
        return self.getp('enabled_point')

    def set_marked_point(self, value, a):
        self.setp('marked_point', value)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)

    def get_marked_point(self, a):
        return self.getp('marked_point')

    def picker_a(self, artist, evt):
        hit = False
        self._figc_hit = -1
        self._figc_hit_pos = (evt.xdata, evt.ydata)
        axes = artist.axes
        if axes is None:
            return False, {}
        x = artist.get_xdata()
        y = artist.get_ydata()
        for k, pd in enumerate(axes.transData.transform(np.transpose(np.vstack((x, y))))):
            if (abs(evt.x - pd[0]) < 5 and
                    abs(evt.y - pd[1]) < 5):
                hit = True
                self._figc_hit = k
                break

        if hit:
            return True, {'child_artist': artist}
        return FigPlot.picker_a(self, artist, evt)

    def dragstart(self, a, evt):
        x = self._artists[0].get_xdata()
        y = self._artists[0].get_ydata()
        self._drag_backup = (x[self._figc_hit],
                             y[self._figc_hit],)
        return 0

    def dragstart_a(self, a, evt):
        return self.dragstart(a, evt)

    def drag_a(self, a, evt, shift=None, scale=None):
        return self.drag(a, evt), scale

    def drag(self, a, evt):
        if evt.inaxes is None:
            return 0
        if evt.xdata is None:
            return 0
        if evt.ydata is None:
            return 0

        x = self._artists[0].get_xdata()
        y = self._artists[0].get_ydata()
        x[self._figc_hit] = evt.xdata
        y[self._figc_hit] = evt.ydata
        self._artists[0].set_xdata(x)
        self._artists[0].set_ydata(y)

        return 1

    def drag_a_get_hl(self, a):
        return self.drag_get_hl(a)

    def drag_get_hl(self, a):
        self._alpha_backup = a.get_alpha()
        for a in self._artists:
            a.set_alpha(0.5)
        return self._artists

    def drag_a_rm_hl(self, a):
        self.drag_rm_hl(a)

    def drag_rm_hl(self, a):
        a.set_alpha(self._alpha_backup)
        for a in self._artists:
            a.set_alpha(self._alpha_backup)

    def dragdone_a(self, a, evt, shift=None, scale=None):
        return self.dragdone(a, evt), scale

    def dragdone(self, a, evt):
        if evt.xdata is None:
            return

        x = self._artists[0].get_xdata()
        y = self._artists[0].get_ydata()
        x[self._figc_hit] = self._drag_backup[0]
        y[self._figc_hit] = self._drag_backup[1]

        x = self._artists[0].get_xdata().copy()
        y = self._artists[0].get_ydata().copy()
        x[self._figc_hit] = evt.xdata
        y[self._figc_hit] = evt.ydata
        action = UndoRedoFigobjMethod(self._artists[0],
                                      'data', (x, y))
        window = evt.guiEvent_memory.GetEventObject().GetTopLevelParent()
        GlobalHistory().get_history(window).make_entry([action])
        return 1

    def get_masked_data(self):
        x = self.getp('x')
        y = self.getp('y')
        flag = self.getp('enabled_point')
        x2 = [x[k] for k in range(len(x)) if flag[k]]
        y2 = [y[k] for k in range(len(x)) if flag[k]]

        ret = [x2, y2]
        if self.getp('xerr') is not None:
            xerr = self.getp('xerr')
            xerr = [xerr[k] for k in range(len(x)) if flag[k]]
            ret.append(xerr)
        if self.getp('yerr') is not None:
            yerr = self.getp('yerr')
            yerr = [yerr[k] for k in range(len(x)) if flag[k]]
            ret.append(yerr)
        return tuple(ret)

    def set_data(self, value, a):
        # this one uses update_artist_request since
        # error bar needs to be changed accordingly.
        #
        # this means that it also needs to overwrite do_artist_update
        # to generte replace_event
        x = value[0]
        y = value[1]
        self.setp('x', x)
        self.setp('y', y)
        self._artists[0].set_xdata(x)
        self._artists[0].set_ydata(y)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)
        self.call_control_changed_callback()

    def get_data(self, a):
        x = self.getp('x').copy()
        y = self.getp('y').copy()
        return (x, y)

    def set_errdata(self, value, a):
        x = value[0]
        y = value[1]
        self.setp('xerr', x)
        self.setp('yerr', y)
        self.set_update_artist_request()
        self.get_figaxes().set_bmp_update(False)

    def get_errdata(self, a):
        x = _copy(self.getp('xerr'))
        y = _copy(self.getp('yerr'))
        return (x, y)

    def prepare_compact_savemode(self):
        var_bk = self._var.copy()
        return var_bk

    def get_mark_data(self):
        vm = self.getp('marked_point')
        l = len(self.getp('enabled_point'))
        if vm is None or len(vm) != l:
            self.setp('marked_point', [False]*l)
            vm = self.getp('marked_point')
        return vm

    def get_export_val(self, a):
        data = super(FigPlotC, self).get_export_val(a)
        data["xdata"] = self.getp('x')
        data["ydata"] = self.getp('y')
        if self.getp('xerr') is not None:
            data["xerrdata"] = self.getp('xerr')
        if self.getp('yerr') is not None:
            data["yerrdata"] = self.getp('yerr')

        return data



from __future__ import print_function
from ifigure.utils.images2gif import writeGif

import os
from ifigure.utils.cbook import image_to_pil
import six
import numpy as np

dither = 1

if six.PY2:
    assert False, "This module does not work with Py2"
    
from PIL import Image

def save_transition(im1, im2, filename='animation.gif',
                    duration=2000, dither=1, steps = 5,
                    twoway_transition = True):
    '''
    im1, im2 : wx.Image
    '''
    iim1 = image_to_pil(im1)
    iim2 = image_to_pil(im2)

    images = []

    if twoway_transition:
        fac = np.hstack((np.linspace(0, 1, steps)[:-1],
                         np.linspace(1, 0, steps)[:-1]))
        print(fac)
        dd = np.array([1] + [0.2]*(steps-2) + [1] + [0.2]*(steps-2))
        print(duration)
    else:
        fac = np.linspace(0, 1, steps)
        dd = np.array([1] + [0.2]*(steps-2) + [1])
        
    for f in fac:
        image = Image.blend(iim1, iim2, f)
        images.append(image)

    duration = duration * dd
    images[0].save(filename,
                   save_all=True,
                   append_images=images[1:],
                   optimize=False, duration=list(duration),
                   loop=0)



import wx
import sys
import time
import wx.aui as aui
import ifigure
import ifigure.events


class HGDiffWindow(wx.MiniFrame):
    def __init__(self, result, *args, **kargs):
        wx.MiniFrame.__init__(self, *args,
                              style=wx.CAPTION |
                              wx.CLOSE_BOX |
                              wx.MINIMIZE_BOX |
                              wx.RESIZE_BORDER |
                              wx.FRAME_FLOAT_ON_PARENT)
        panel = wx.Panel(self)
        vbox = wx.BoxSizer(wx.VERTICAL)
        self.SetSizer(vbox)
        vbox.Add(panel, 1, wx.EXPAND | wx.ALL, 5)

        self.nb = aui.AuiNotebook(panel)
        bpanel = wx.Panel(panel)
        button = wx.Button(bpanel, wx.ID_ANY, "Done")
        vbox = wx.BoxSizer(wx.VERTICAL)
        hbox = wx.BoxSizer(wx.HORIZONTAL)

        panel.SetSizer(vbox)
        vbox.Add(self.nb, 1, wx.EXPAND)
        vbox.Add(bpanel, 0, wx.EXPAND | wx.ALL, 5)

        hbox.AddStretchSpacer()
        hbox.Add(button, 0, wx.EXPAND | wx.ALL, 2)
        bpanel.SetSizer(hbox)

        #sys.stdout = RedirectOutput(self.log)
        #sys.stderr = RedirectOutput(self.log)

        for k, r in enumerate(result):
            log = wx.TextCtrl(self.nb, -1,
                              style=wx.TE_MULTILINE | wx.TE_READONLY)
            log.AppendText(r['diff'])
            self.nb.AddPage(log,  r['filename'])

        button.Bind(wx.EVT_BUTTON, self.onClose)
        self.Show()
        self.SetTitle('HG diff result')

    def onClose(self, evt):
        self.Close()
        import ifigure.mto.hg_support
        ifigure.mto.hg_support.diffwindow = None



from __future__ import print_function
import ifigure.utils.debug as debug
from ifigure.widgets.undo_redo_history import UndoRedoAddRemoveArtists, GlobalHistory
import ifigure.widgets.canvas.custom_picker as cpicke
from ifigure.utils.args_parser import ArgsParser
import ifigure.utils.cbook as cbook
from ifigure.mto.axis_user import XUser, YUser, ZUser, CUser
from ifigure.mto.fig_obj import FigObj, mask_negative
import ifigure.events
import ifigure
import weakref
import wx
import os
import sys
import logging
import numpy as np

from matplotlib.cm import ScalarMappable
from matplotlib.colors import ColorConverter
cc = ColorConverter()


dprint1, dprint2, dprint3 = debug.init_dprints('FigQuiver')

default_kargs = {'alpha':  1.0,
                 'pivot':  'tail',
                 'cmap':  'jet'}


class FigQuiver(FigObj, XUser, YUser, CUser):
    def __new__(cls, *args, **kywds):
        """
        quiver : quiver plot 
        quiver(U, V, **kargs)
        quiver(U, V, C, **kargs)
        quiver(X, Y, U, V, **kargs)
        quiver(X, Y, U, V, C, **kargs)
        """
        def set_hidden_vars(obj):
            return obj

        if 'src' in kywds:
            obj = FigObj.__new__(cls, *args, **kywds)
            obj = set_hidden_vars(obj)
            return obj
        p = ArgsParser()
        p.add_opt('x', None, ['numbers|nonstr', 'dynamic'])
        p.add_opt('y', None, ['numbers|nonstr', 'dynamic'])
        p.add_var('u', ['numbers|nonstr', 'dynamic'])
        p.add_var('v', ['numbers|nonstr', 'dynamic'])
        p.add_opt('c', None, ['numbers|nonstr', 'dynamic'])

        p.set_default_list(default_kargs)
        p.add_key2(("alpha", "cmap"))
        p.add_key2(("pivot", "pivot"))
        p.set_pair("x", "y")  # x and y should be given
        # together
        p.set_ndconvert("x", "y", "u", "v", "c")
#        p.set_squeeze_minimum_1D("x","y","z")

        v, kywds, d, flag = p.process(*args, **kywds)
        dprint2(d)

        if not flag:
            raise ValueError('Failed when processing argument')
            return None

        obj = FigObj.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)

        for name in ('u', 'v', 'c', 'x', 'y'):
            obj.setvar(name, v[name])
        for name in ('alpha', 'cmap', 'pivot'):
            obj.setvar(name, v[name])

        if v['cmap'] is not None:
            kywds['cmap'] = v['cmap']

        obj.setvar("kywds", kywds)
        return obj

    def __init__(self, *args, **kywds):
        self._data_extent = None
        XUser.__init__(self)
        YUser.__init__(self)
        CUser.__init__(self)

        self._pick_pos = None
        args = []
        if 'src' not in kywds:
            kywds = self.getvar("kywds")
        super(FigQuiver, self).__init__(**kywds)

    @classmethod
    def isFigQuiver(self):
        return True

    @classmethod
    def can_have_child(self, child=None):
        return False

    @classmethod
    def get_namebase(self):
        return 'quiver'

    @classmethod
    def load_classimage(self):
        from ifigure.ifigure_config import icondir as path
        idx1 = cbook.LoadImageFile(path, 'plot1.png')
        return [idx1]

    @classmethod
    def attr_in_file(self):
        return (["alpha", "pivot"] +
                super(FigQuiver, self).attr_in_file())

    def property_in_file(self):
        return (['facecolor', 'edgecolor',  'elinewidth', 'alpha'] +
                super(FigQuiver, self).property_in_file())

    @classmethod
    def property_in_palette(self):
        return (['quiver', 'path', 'patch'], [
            ["qheadlength", "qheadwidth", "qpivot",
             "alpha_2"],
            ["pedgecolor_2", "elinewidth"],
            ["facecolor_2"]])

    def set_parent(self, parent):
        XUser.unset_ax(self)
        YUser.unset_ay(self)
        CUser.unset_ac(self)
        super(FigQuiver, self).set_parent(parent)
        XUser.get_xaxisparam(self)
        YUser.get_yaxisparam(self)
        CUser.get_caxisparam(self)

    def args2var(self):
        ret = self._args2var()
        if ret:
            if 'cmap' in self.getvar("kywds"):
                cax = self.get_caxisparam()
                cax.set_cmap(self.getvar('kywds')['cmap'])
        return ret

    def _args2var(self):
        names0 = self.attr_in_file()
        names = ["x", "y", "u", "v", "c"] + names0
        use_np = [True]*5 + [False]*(len(names0))
        # n and v can be dynamic too.
        values = list(self.put_args2var(names,
                                        use_np))
        if values[0] is None:
            u = values[2]
            if len(u.shape) != 2:
                return False
            x = np.arange(u.shape[1])
            y = np.arange(u.shape[0])
            values[0], values[1] = np.meshgrid(x, y)

        self.setp('x', values[0])
        self.setp('y', values[1])
        self.setp('u', values[2])
        self.setp('v', values[3])
        self.setp('c', values[4])
        return True

    def _eval_xyz(self):
        if self.getp('use_var'):
            success = self.handle_use_var()
            if not success:
                return None, None, None, None, None
        return self.getp(("x", "y", "u", "v", "c"))

    def generate_artist(self):
        # this method generate artist
        # if artist does exist, update artist
        # based on the information specifically
        # managed by fig_obj tree. Any property
        # internally managed by matplotlib
        # does not change
        container = self.get_container()
        if container is None:
            return

        flag = 0
        if self.isempty() is False:
            return
        x, y, u, v, c = self._eval_xyz()  # this handles "use_var"
        if u is None:
            return
        if v is None:
            return

        lp = self.getp("loaded_property")

#           v, n, FillMode = self.getp(('v', 'n', 'FillMode'))

        kywds = self.getvar('kywds')
        kywds['alpha'] = self.getp('alpha')

        args = []
        if (x is not None and y is not None):
            args.append(x)
            args.append(y)
        args.append(u)
        args.append(v)
        if c is not None:
            args.append(c)
        cax = self.get_caxisparam()

        dprint2(args)
        if len(args) == 0:
            return

        self._data_extent = [np.min(x), np.max(x), np.min(y), np.max(y)]

        try:
            a = container.quiver(*args, **kywds)
            self._artists = [a]
            a.figobj = self
            a.figobj_hl = []
            a.set_zorder(self.getp('zorder'))

            if c is not None:
                self._mappable = self._artsits
            for a in self.get_mappable():
                cax.set_crangeparam_to_artist(a)
        except Exception:
            logging.exception(
                "FigQuiver:generate_artist : artist generation failed")

        if lp is not None:
            #              print lp
            for i, var in enumerate(lp):
                if len(self._artists) > i:
                    self.set_artist_property(self._artists[i], var)
            self.delp("loaded_property")

    def del_artist(self, artist=None, delall=False):
        #
        #  delete highlight and artist
        #  for fig_contour, all artists are treated
        #  as one set. it does not support to delete
        #  a part of path collections
        delall = True
        if delall:
            artistlist = self._artists
        else:
            artistlist = self._artists

        self.store_loaded_property()
#        if not self.hasp("loaded_property"):
#             self.load_data2(self.save_data2({}))
#        self.highlight_artist(False)
        for a in artistlist:
            a.remove()
        self._mappable = None
        super(FigQuiver, self).del_artist(artistlist)

    def get_mappable(self):
        if self.getp('c') is not None:
            return self._artists
        else:
            return []

    def reset_artist(self):
        #        print('resetting contour artist')
        self.del_artist(delall=True)
# (why not)  self.delp('loaded_property')
        self.setp('use_var', True)
        self.generate_artist()

    def highlight_artist(self, val, artist=None):
        if artist is None:
            alist = self._artists
        else:
            alist = artist

        if val == True:
            if self._parent is None:
                return
            container = self.get_container()
            if container is None:
                return

            try:
                x = self.getp('x').flatten()
                y = self.getp('y').flatten()
            except:
                return
            hl = container.plot(x, y, marker='s',
                                color='k', linestyle='None',
                                markerfacecolor='None',
                                markeredgewidth=0.5,
                                scalex=False, scaley=False)
            for item in hl:
                alist[0].figobj_hl.append(item)
        else:
            for a in alist:
                if len(a.figobj_hl) == 0:
                    continue
                for hl in a.figobj_hl:
                    hl.remove()
                a.figobj_hl = []

#
#   HitTest
#
    def picker_a(self, artist, evt):
        if artist.contains(evt)[0]:
            return True, {'child_artist': artist}
        return False, {}

    def picker_a0(self, artist, evt):
        hit, extra = self.picker_a(artist, evt)
        if hit:
            return hit, extra, 'area', 3
        else:
            return False, {}, None, 0
    #
    #  getter/setter
    #

    def set_alpha(self, value, a):
        for a in self._artists:
            a.set_alpha(value)

    def get_alpha(self, a):
        v = self._artists[0].get_alpha()
        return v

    def set_edgecolor(self, value, a):
        for a in self._artists:
            a.set_edgecolor(value)

    def get_edgecolor(self, a):
        v = a.get_edgecolor()
        return v[0]

    def set_facecolor(self, value, a):
        for a in self._artists:
            a.set_facecolor(value)

    def get_facecolor(self, a):
        v = a.get_facecolor()
        return v[0]

    def set_elinewidth(self, value, a):
        for a in self._artists:
            a.set_linewidth(value)

    def get_elinewidth(self, a):
        v = a.get_linewidth()
        if len(v) > 0:
            return v[0]
        return ['1.0']

    def set_angles(self, v, a):
        a.angles = str(v)
        a._new_UV = True

    def set_headlength(self, v, a):
        a.headlength = float(v)
        a._new_UV = True

    def set_headaxislength(self, v, a):
        a.headaxislength = float(v)
        a._new_UV = True

    def set_headwidth(self, v, a):
        a.headwidth = float(v)
        a._new_UV = True

    def set_pivot(self, v, a):
        return self.setp('pivot', v)

    def get_angles(self, a):
        return a.angles

    def get_headlength(self, a):
        return a.headlength

    def get_headaxislength(self, a):
        return a.headaxislength

    def get_headwidth(self,  a):
        return a.headwidth

    def get_pivot(self, a):
        return self.getp('pivot')

    def get_xrange(self, xrange=[None, None], scale='linear'):
        x, y, u, v, c = self._eval_xyz()  # this handles "use_var"
        if x is None:
            return
        if scale == 'log':
            x = mask_negative(x)
        return self._update_range(xrange, [np.nanmin(x), np.nanmax(x)])

    def get_yrange(self, yrange=[None, None],
                   xrange=[None, None], scale='linear'):
        #        de = self.get_data_extent()
        x, y, u, v, c = self._eval_xyz()  # this handles "use_var"
        if x is None:
            return
        if y is None:
            return
        if scale == 'log':
            y = mask_negative(y)
        return self._update_range(yrange, (np.nanmin(y), np.nanmax(y)))

    def get_crange(self, crange=[None, None],
                   xrange=[None, None],
                   yrange=[None, None], scale='linear'):

        x, y, u, v, c = self._eval_xyz()  # this handles "use_var"
        if c is None:
            return crange
        return crange

        ###
        if (x is None or
                y is None):
            x = np.arange(z.shape[1])
            y = np.arange(z.shape[0])

        if (xrange[0] is not None and
            xrange[1] is not None and
            yrange[0] is not None and
                yrange[1] is not None):
            zt = np.ma.masked_array(z)
            zt[(y < yrange[0]) | (y > yrange[1]), :] = np.ma.masked
            zt[:, (x < xrange[0]) | (x > xrange[1])] = np.ma.masked
            if scale == 'log':
                zt[z <= 0] = np.ma.masked
            crange = self._update_range(crange, (np.amin(zt), np.amax(zt)))
        return crange

    def save_data2(self, data=None):
        def check(obj, name):
            if not isinstance(obj.getp(name), np.ndarray):
                return False
            if not isinstance(obj.getvar(name), np.ndarray):
                return False
            return obj.getp(name) is obj.getvar(name)

        if data is None:
            data = {}
        var = {'x': check(self, 'x'),
               'y': check(self, 'y'),
               'u': check(self, 'u'),
               'v': check(self, 'v'),
               'c': check(self, 'c')}

        if not var["x"]:
            if self._save_mode == 0:
                var["xdata"] = self.getp("x")
            else:
                var["xdata"] = np.array([[0, 1], [0, 1]])
        if not var["y"]:
            if self._save_mode == 0:
                var["ydata"] = self.getp("y")
            else:
                var["ydata"] = np.array([[0, 0], [1, 1]])
        if not var["u"]:
            if self._save_mode == 0:
                var["udata"] = self.getp("u")
            else:
                var["udata"] = np.array([[1, 1], [1, 1]])
        if not var["v"]:
            if self._save_mode == 0:
                var["vdata"] = self.getp("v")
            else:
                var["vdata"] = np.array([[1, 1], [1, 1]])
        if not var["c"]:
            if self._save_mode == 0:
                var["cdata"] = self.getp("c")
            else:
                var["cdata"] = np.array([[1, 1], [1, 1]])

        data['FigQuiver'] = (1, var)
        data = super(FigQuiver, self).save_data2(data)
        return data

    def load_data2(self, data):
        d = data['FigQuiver']
        super(FigQuiver, self).load_data2(data)
        dprint2('load_data2', d[1])
        var = d[1]
        names = ["x", "y", "u", "v", "c"]
        for name in names:
            if var[name]:
                self.setp(name,
                          self.getvar(name))
            else:
                self.setp(name, var[name+'data'])


class FigQuiver3D(FigQuiver, ZUser):
    def __new__(cls, *args, **kywds):
        """
        quiver(X, Y, Z, U, V, W, **kwargs)
        """
        def set_hidden_vars(obj):
            return obj

        if 'src' in kywds:
            obj = FigObj.__new__(cls, *args, **kywds)
            obj = set_hidden_vars(obj)
            return obj

        if 'cz' in kywds and kywds['cz']:
            def_alpha = None
            def_ec = None
            def_fc = None
            def_lw = 0.0
        else:
            def_alpha = 1.0
            def_ec = (0, 0, 0, 1)
            def_fc = (0, 0, 1, 1)
            def_lw = 1.0

        p = ArgsParser()
        p.add_var('x', ['numbers|nonstr', 'dynamic'])
        p.add_var('y', ['numbers|nonstr', 'dynamic'])
        p.add_var('z', ['numbers|nonstr', 'dynamic'])
        p.add_var('u', ['numbers|nonstr', 'dynamic'])
        p.add_var('v', ['numbers|nonstr', 'dynamic'])
        p.add_var('w', ['numbers|nonstr', 'dynamic'])
        p.add_key('cz', False, 'bool')
        p.add_key('cdata', None)
        p.set_default_list(default_kargs)
        p.add_key2(("cmap"))
        p.add_key2(("pivot", "pivot"))
        p.add_key("length", 1.0)
        p.add_key("arrow_length_ratio", 0.3)
        p.add_key('alpha', def_alpha)
        p.add_key('facecolor', def_fc)
        p.add_key('edgecolor', def_ec)

        p.set_ndconvert("x", "y", "z", "u", "v", "w", "c")

        v, kywds, d, flag = p.process(*args, **kywds)
        dprint2(d)

        if not flag:
            raise ValueError('Failed when processing argument')
            return None

        obj = FigObj.__new__(cls, *args, **kywds)
        obj = set_hidden_vars(obj)

        for name in ('u', 'v', 'w', 'x', 'y', 'z'):
            obj.setvar(name, v[name])
        for name in ('alpha', 'cmap', 'pivot', 'length', 'arrow_length_ratio',
                     'cz', 'cdata', 'facecolor', 'edgecolor'):
            obj.setvar(name, v[name])

        if v['cmap'] is not None:
            kywds['cmap'] = v['cmap']

        obj.setvar("kywds", kywds)
        return obj

    def __init__(self, *args, **kywds):
        ZUser.__init__(self)
        super(FigQuiver3D, self).__init__(**kywds)

    @classmethod
    def get_namebase(self):
        return 'quiver3d'

    @classmethod
    def attr_in_file(self):
        return (["length", "arrow_length_ratio", "facecolor", "edgecolor",
                 "linewidth", ] +
                super(FigQuiver3D, self).attr_in_file())

    @classmethod
    def property_in_palette(self):
        return (['quiver', 'path/patch'], [
            ["qpivot", "q3dlength", "q3dratio", "alpha_2"],
            ["facecolor_2", "edgecolor_2", "elinewidth"], ])

    def set_parent(self, parent):
        ZUser.unset_az(self)
        super(FigQuiver3D, self).set_parent(parent)
        ZUser.get_zaxisparam(self)

    def _args2var(self):
        names0 = self.attr_in_file()
        names = ["x", "y", "z", "u", "v", "w"] + names0
        use_np = [True]*6 + [False]*(len(names0))
        # n and v can be dynamic too.
        values = list(self.put_args2var(names,
                                        use_np))
        if values[0] is None:
            u = values[2]
            if len(u.shape) != 2:
                return False
            x = np.arange(u.shape[1])
            y = np.arange(u.shape[0])
            values[0], values[1] = np.meshgrid(x, y)

        self.setp('x', values[0])
        self.setp('y', values[1])
        self.setp('z', values[2])
        self.setp('u', values[3])
        self.setp('v', values[4])
        self.setp('w', values[5])
        return True

    def _eval_xyzw(self):
        if self.getp('use_var'):
            success = self.handle_use_var()
            if not success:
                return None, None, None, None, None
        return self.getp(("x", "y", "z", "u", "v", "w"))

    def generate_artist(self):
        # this method generate artist
        # if artist does exist, update artist
        # based on the information specifically
        # managed by fig_obj tree. Any property
        # internally managed by matplotlib
        # does not change
        container = self.get_container()
        if container is None:
            return
        if not self.get_figaxes().get_3d():
            return
        flag = 0
        if self.isempty() is False:
            return
        x, y, z, u, v, w = self._eval_xyzw()  # this handles "use_var"
        if u is None:
            return
        if v is None:
            return
        if w is None:
            return

        lp = self.getp("loaded_property")

#           v, n, FillMode = self.getp(('v', 'n', 'FillMode'))

        kywds = self.getvar('kywds')
        kywds['alpha'] = self.getp('alpha')
        kywds['length'] = self.getp('length')
        kywds['pivot'] = self.getp('pivot')
        kywds['arrow_length_ratio'] = self.getp('arrow_length_ratio')

        kywds['alpha'] = self.getp('alpha') if self.getp(
            'alpha') is not None else 1

        fc = self.getp('facecolor')
        if isinstance(fc, str):
            fc = cc.to_rgba(fc)
        if fc is None:
            fc = [0, 0, 0, 0]
        else:
            fc = list(fc)
            if self.getp('alpha') is not None:
                fc[3] = self.getp('alpha')
        ec = self.getp('edgecolor')
        if isinstance(ec, str):
            ec = cc.to_rgba(ec)
        if ec is None:
            ec = [0, 0, 0, 0]
        else:
            ec = list(ec)
            if self.getp('alpha') is not None:
                ec[3] = self.getp('alpha')
        if self.getvar('cz'):
            kywds['cz'] = self.getvar('cz')
            if self.getvar('cdata') is not None:
                cdata = self.getvar('cdata')
            else:
                cdata = z
            if np.iscomplexobj(cdata):
                kywds['facecolordata'] = cdata.real
            else:
                kywds['facecolordata'] = cdata
        else:
            kywds['facecolor'] = (fc,)
        kywds['edgecolor'] = (ec,)
        kywds['linewidths'] = 0.0 if self.getp(
            'linewidth') is None else self.getp('linewidth')

        args = (x, y, z, u, v, w)
        cax = self.get_caxisparam()

        dprint2(args)
        if len(args) == 0:
            return

        self._data_extent = [np.min(x), np.max(x), np.min(y), np.max(y)]

        try:
            a = container.quiver(*args, **kywds)
            self._artists = [a]
            a.figobj = self
            a.figobj_hl = []
            a.set_zorder(self.getp('zorder'))

            for a in self.get_mappable():
                cax.set_crangeparam_to_artist(a)
        except Exception:
            logging.exception(
                "FigQuiver3D:generate_artist : artist generation failed")

        if lp is not None:
            # print lp
            for i, var in enumerate(lp):
                if len(self._artists) > i:
                    self.set_artist_property(self._artists[i], var)
            self.delp("loaded_property")

    def highlight_artist(self, val, artist=None):
        if artist is None:
            alist = self._artists
        else:
            alist = artist

        if val == True:
            if self._parent is None:
                return
            container = self.get_container()
            if container is None:
                return

            try:
                x = self.getp('x').flatten()
                y = self.getp('y').flatten()
                z = self.getp('z').flatten()
            except:
                return
            hl = container.plot(x, y, z, marker='s',
                                color='k', linestyle='None',
                                markerfacecolor='None',
                                markeredgewidth=0.5,
                                scalex=False, scaley=False)
            for item in hl:
                alist[0].figobj_hl.append(item)
        else:
            for a in alist:
                if len(a.figobj_hl) == 0:
                    continue
                for hl in a.figobj_hl:
                    hl.remove()
                a.figobj_hl = []

    def get_mappable(self):
        if self.getvar('cz'):
            return [a for a in self._artists if isinstance(a, ScalarMappable)]
        else:
            return []

    def set_edgecolor(self, value, a):
        for a in self._artists:
            a.set_edgecolor(value)
            a._update_ec = True
            print('update_ec')

    def _update_artist(self):
        self.highlight_artist(False)
        self.del_artist(delall=True)
        # self.delp('loaded_property')
        self.generate_artist()
        sel = [weakref.ref(self._artists[0])]
        import wx
        app = wx.GetApp().TopWindow
        ifigure.events.SendSelectionEvent(self, w=app, selections=sel)

    def set_pivot(self, v, a):
        self.setp('pivot', v)
        self._update_artist()

    def set_edgecolor(self, value, a):
        self.setp('edgecolor', value)
        a.set_edgecolor([value])

    def get_edgecolor(self, a=None):
        return self.getp('edgecolor')

    def set_facecolor(self, value, a):
        if self.getvar('cz'):
            return
        if value == 'disabled':
            return
        if isinstance(value, str):
            value = cc.to_rgba(value)
        alpha = self.getp('alpha')
        if alpha is None:
            alpha = 1.0
        value = (value[0], value[1], value[2], alpha)
        self.setp('facecolor', value)
        a.set_facecolor([value])

    def get_facecolor(self, a=None):
        if self.getvar('cz'):
            return 'disabled'
        return self.getp('facecolor')

    def set_q3dlength(self, v, a):
        self.setp('length', float(v))
        self._update_artist()

    def get_q3dlength(self, a):
        return self.getp('length')

    def set_q3dratio(self, v, a):
        self.setp('arrow_length_ratio', float(v))
        self._update_artist()

    def get_q3dratio(self, a):
        return self.getp('arrow_length_ratio')

    #
    #  xyzc-range
    #

    def get_xrange(self, xrange=[None, None], scale='linear'):
        x, y, z, u, v, w = self._eval_xyzw()  # this handles "use_var"
        if x is None:
            return
        if scale == 'log':
            x = mask_negative(x)
        return self._update_range(xrange, [np.nanmin(x), np.nanmax(x)])

    def get_yrange(self, yrange=[None, None],
                   xrange=[None, None], scale='linear'):
        #        de = self.get_data_extent()
        x, y, z, u, v, w = self._eval_xyzw()  # this handles "use_var"
        if y is None:
            return
        if scale == 'log':
            y = mask_negative(y)
        return self._update_range(yrange, (np.nanmin(y), np.nanmax(y)))

    def get_zrange(self, zrange=[None, None],
                   xrange=[None, None],
                   yrange=[None, None], scale='linear'):
        x, y, z, u, v, w = self._eval_xyzw()  # this handles "use_var"
        if z is None:
            return
        if scale == 'log':
            z = mask_negative(z)
        return self._update_range(zrange, (np.nanmin(z), np.nanmax(z)))

    def get_crange(self, crange=[None, None],
                   xrange=[None, None],
                   yrange=[None, None],
                   scale='linear'):
        cdata = self.getvar('cdata')
        cz = self.getvar('cz')
        if not cz:
            return crange
        if cdata is None:
            x, y, z, u, v, w = self._eval_xyzw()  # this handles "use_var"
            crange = self._update_range(crange,
                                        (np.nanmin(z), np.nanmax(z)))

        else:
            if np.iscomplexobj(cdata):
                tmp = np.max(np.abs(cdata))
                crange = self._update_range(crange,
                                            (-tmp, tmp))

            else:
                crange = self._update_range(crange,
                                            (np.nanmin(cdata), np.nanmax(cdata)))

        return crange

    def save_data2(self, data=None):
        def check(obj, name):
            if not isinstance(obj.getp(name), np.ndarray):
                return False
            if not isinstance(obj.getvar(name), np.ndarray):
                return False
            return obj.getp(name) is obj.getvar(name)

        if data is None:
            data = {}
        var = {'z': check(self, 'z'),
               'w': check(self, 'w')}
        if not var["z"]:
            if self._save_mode == 0:
                var["zdata"] = self.getp("z")
            else:
                var["zdata"] = np.array([[0, 1], [0, 1]])
        if not var["w"]:
            if self._save_mode == 0:
                var["wdata"] = self.getp("w")
            else:
                var["ydata"] = np.array([[0, 0], [1, 1]])

        data['FigQuiver3D'] = (1, var)
        data = super(FigQuiver3D, self).save_data2(data)
        return data

    def load_data2(self, data):
        d = data['FigQuiver3D']
        super(FigQuiver3D, self).load_data2(data)
        dprint2('load_data2', d[1])
        var = d[1]
        names = ["w", "z"]
        for name in names:
            if var[name]:
                self.setp(name,
                          self.getvar(name))
            else:
                self.setp(name, var[name+'data'])



from __future__ import print_function
import os
import time
print(" ")
print(" ##########################")
print(" Running GENRAY-Loki Script")
print(" ##########################")
print("")
print(" Running evniroment for script file")
# print "   proj    =", proj
print(("   treeobj =", obj.get_full_path(), obj))
print(" ")

print((" Script File : ",  obj.path2fullpath()))
print((" Last Modified : ", time.ctime(obj._script._script_mtime)))

solver = obj.get_parent()
print(('model folder:', solver.model.get_full_path()))
print(('solution folder :', solver.sol.get_full_path()))

root = obj.get_root_parent()
print(('setting:', root.psetting.get_full_path()))
print(('connection:', root.psetting.loki.get_full_path()))

proj = obj.get_root_parent()
wdir = proj.eval("wdir")
runname = os.path.basename(proj.eval("filename"))[:-4]

gin_file = os.path.join(wdir, 'genray.in')
cqlin_file = os.path.join(wdir, 'cqlinput')
cqlpbs = os.path.join(wdir, 'cql3d.pbs')
gfile = 'g111.1111'


gfile_full = os.path.join(wdir, gfile)

mto = solver.model.GENRAY

mto.genray_input.tokamak.setvar("eqdskin", ['"'+gfile+'"'])
mto.cql_input.eqsetup.setvar("eqdskin", ['"'+gfile+'"'])

mto.gfile.call_method('onWriteFile', file=gfile_full)
mto.genray_input.call_method('onWriteFile', file=gin_file)
mto.cql_input.call_method('onWriteFile', file=cqlin_file)
mto.cql3dpbs.call_method('onWriteFile', file=cqlpbs)

loki = root.psetting.loki
rdir = os.path.basename(wdir)
rdir = str(os.path.join('py_genray', runname))
loki.call_method('onSend', file=gfile_full, rdir=rdir)
loki.call_method('onSend', file=gin_file, rdir=rdir)
loki.call_method('onSend', file=cqlin_file, rdir=rdir)
loki.call_method('onSend', file=cqlpbs, rdir=rdir)

command = 'cd '+rdir + ';' + '/home/shiraiwa/bin/launch_xgenray -0'
print('Now calling genray')
loki.call_method('onExec',  command=command)
print('Now submitting CQL3D job')
command = 'cd '+rdir + ';' + 'qsub cql3d.pbs'
loki.call_method('onExec',  command=command)

# loki.onSend(rdir='/home/shiraiwa/xxx')
